#ifndef __SIMPLE_PTR_LIST_H__
#define __SIMPLE_PTR_LIST_H__

struct TPlex
{
	TPlex* pNext;
	void *data() { return this+1; }

	static TPlex* Create(TPlex*& pHead, UINT nMax, UINT cbElement)
	{
		ASSERT(nMax > 0 && cbElement > 0);

//		TPlex* p = (TPlex*) new BYTE[sizeof(TPlex) + nMax * cbElement];
		DWORD cbSize = sizeof(TPlex) + nMax * cbElement;
		TPlex* p = (TPlex*) new BYTE[cbSize];
		memset(p, 0, cbSize);

		// may throw exception
		p->pNext = pHead;
		pHead = p;  // change head (adds in reverse order for simplicity)
		return p;
	}

	void FreeDataChain()	// free this one and links
	{
		TPlex* p = this;
		while(p != NULL)
		{
			BYTE* bytes = (BYTE*) p;
			TPlex* pNext = p->pNext;
			delete[] bytes;
			p = pNext;
		}
	}
};

///////////////////////////////////////////////////////////////////////////////
// Simple Linked Pointer List

template<class TYPE, class ARG_TYPE>
class CSimplePtrList
{
protected:
	struct TNode
	{
		TNode* pNext;
		TNode* pPrev;
		TYPE data;
	};

public:
	CSimplePtrList()
	{
		m_nCount = 0;
		m_pNodeHead = m_pNodeTail = NULL;
	}
	~CSimplePtrList()
	{
		RemoveAll();
	}

	void RemoveAll()
	{
		TNode* pNextNode;
		for(TNode* pNode = m_pNodeHead; pNode; pNode = pNextNode)
		{
			pNextNode = pNode->pNext;
			delete pNode;
		}
	}

	inline int GetCount() const
	{
		return m_nCount;
	}

	inline POSITION GetHeadPosition() const
	{
		return (POSITION) m_pNodeHead;
	}

	TYPE GetNext(POSITION& rPosition) const
	{
		TNode* pNode = (TNode*) rPosition;
		rPosition = (POSITION) pNode->pNext;
		return pNode->data;
	}

	POSITION AddTail(ARG_TYPE newElement)
	{
		TNode* pNewNode = new TNode;

		pNewNode->pPrev = m_pNodeTail;
		pNewNode->pNext = NULL;
		pNewNode->data = newElement;

		if(m_pNodeTail != NULL)
			m_pNodeTail->pNext = pNewNode;
		else
			m_pNodeHead = pNewNode;
		m_pNodeTail = pNewNode;
		m_nCount++;

		return (POSITION) pNewNode;
	}

private:
	TNode* m_pNodeHead;
	TNode* m_pNodeTail;
	int m_nCount;
};

#endif
