#include "stdafx.h"
#include "xfc.h"

CMenu::CMenu()
{
	m_hMenu = CreateMenu();
}

CMenu::~CMenu()
{
	DestroyMenu(m_hMenu);
}

CMenu* CMenu::GetSubMenu(int nPos) const
{
	return NULL;
}

BOOL CMenu::AppendMenu(UINT nFlags, UINT nID, LPCTSTR pszLabel)
{
	return ::AppendMenu(m_hMenu, nFlags, nID, pszLabel);
}

UINT CMenu::CheckMenuItem(UINT nIDCheckItem, UINT nCheck)
{
	return ::CheckMenuItem(m_hMenu, nIDCheckItem, nCheck);
}

UINT CMenu::EnableMenuItem(UINT nIDEnableItem, UINT nEnable)
{
	return ::EnableMenuItem(m_hMenu, nIDEnableItem, nEnable);
}

BOOL CMenu::ModifyMenu(UINT nPosition, UINT nFlags, UINT nID, LPCTSTR pszLabel)
{
	return ::ModifyMenu(m_hMenu, nPosition, nFlags, nID, pszLabel);
}

void CMenu::TrackPopupMenu(UINT nFlags, int x, int y, CWnd* pWnd)
{
	::TrackPopupMenu(m_hMenu, nFlags, x, y, 0, pWnd->GetSafeHwnd(), NULL);
}

/////////////////////////////////////////////////////////////////////////////////////////////

BOOL CRect::PtInRect(POINT point) const
{
	return ::PtInRect((RECT*)this, point);
}

BOOL CRect::PtInRect(int x, int y) const
{
	POINT point = { x, y };
	return ::PtInRect(this, point);	
}

BOOL CRect::EqualRect(RECT* pRect) const
{
	return ::EqualRect(this, pRect);
}

void CRect::CopyRect(RECT* pSrcRect)
{
	::CopyRect(this, pSrcRect);
}

void CRect::SetRect(int x1, int y1, int x2, int y2)
{
	::SetRect(this, x1, y1, x2, y2);
}

void CRect::InflateRect(int x, int y)
{
	::InflateRect(this, x, y);
}

/////////////////////////////////////////////////////////////////////////////////////////////

CStatusBar::CStatusBar()
{
	m_nPaneCount = 0;
	m_pPaneInfos = NULL;
	m_pRightEdges = NULL;
}

CStatusBar::~CStatusBar()
{
	if(m_pPaneInfos != NULL)
	{
		delete [] m_pPaneInfos;
		delete [] m_pRightEdges;
	}
}

BOOL CStatusBar::Create(CFrameWnd* pParentWnd, DWORD dwStyle, UINT nID)
{
	m_hWnd = CreateStatusWindow(
		WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | CCS_BOTTOM,
		"Ready", pParentWnd->GetSafeHwnd(), nID);

	if(m_hWnd != NULL)
	{
		pParentWnd->m_pStatusBar = this;
		return TRUE;
	}

	return FALSE;	
}

BOOL CStatusBar::SetIndicators(const UINT *pIDArray, int nIDCount)
{
	if(m_nPaneCount < nIDCount && m_pPaneInfos != NULL)
	{
		delete [] m_pPaneInfos;
		delete [] m_pRightEdges;
	}
	
	m_nPaneCount = nIDCount;
	m_pPaneInfos = new CStbPaneInfo[nIDCount];
	m_pRightEdges = new int[nIDCount];

	for(int n=0; n<nIDCount; n++)
	{
		m_pPaneInfos[n].cxWidth = 0;
		m_pRightEdges[n] = 0;
	}

	SendMessage(m_hWnd, SB_SETPARTS, 5, (LPARAM) pIDArray);

	return TRUE;
}

void CStatusBar::SetPaneInfo(int nIndex, UINT nID, UINT nStyle, int cxWidth)
{
	if(nIndex >= m_nPaneCount)
		return;

	m_pPaneInfos[nIndex].cxWidth = cxWidth;
}

BOOL CStatusBar::SetPaneText(int nIndex, LPCTSTR pszNewText, BOOL bUpdate)
{
	return TRUE;
}

void CStatusBar::SetPaneStyle(int nIndex, UINT nStyle)
{
}

// Positioning	
int CStatusBar::GetHeight() const
{
	RECT rcBar;

	GetClientRect(m_hWnd, &rcBar);
	return rcBar.bottom;
}

void CStatusBar::SetWidth(int cx)
{
	m_pRightEdges[m_nPaneCount -1] = -1;
	for(int n=(m_nPaneCount-2); n>=0; n--)
	{
		cx -= m_pPaneInfos[n+1].cxWidth;
		m_pRightEdges[n] = cx;
	}

	SendMessage(m_hWnd, SB_SETPARTS, m_nPaneCount, (LPARAM) m_pRightEdges);
}

/////////////////////////////////////////////////////////////////////////////////////////////

CToolBar::CToolBar()
{
}

CToolBar::~CToolBar()
{
}

BOOL CToolBar::Create(CFrameWnd* pParentWnd, DWORD dwStyle, UINT nID)
{
	m_hWnd = CreateWindow(TOOLBARCLASSNAME, "",
		dwStyle | WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS | CCS_NORESIZE,
		0,0,100,24, pParentWnd->GetSafeHwnd(), NULL, XfcGetHinstance(), this);

	if(m_hWnd == NULL) return FALSE;

	pParentWnd->m_pToolBar = this;
	SendMessage(m_hWnd, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0);
	return TRUE;
}

BOOL CToolBar::LoadBitmap(UINT nResID)
{
	HBITMAP hbmNew;

	hbmNew = ::LoadBitmap(XfcGetHinstance(), MAKEINTRESOURCE(nResID));

	if(1)
	{
		TBADDBITMAP addBitmap;

		addBitmap.hInst = NULL; //XfcGetHinstance();
		addBitmap.nID = (UINT) hbmNew;

		return (BOOL) (SendMessage(m_hWnd, TB_ADDBITMAP, 8, (LPARAM)&addBitmap) != -1);
	}
	else
	{
	TBREPLACEBITMAP replaceBitmap;

	replaceBitmap.hInstOld = NULL;
	replaceBitmap.nIDOld = 0;
	replaceBitmap.hInstNew = NULL;
	replaceBitmap.nIDNew = (UINT) hbmNew;
	replaceBitmap.nButtons = 7;//bitmap.bmWidth / m_sizeImage.cx;

	return (BOOL) SendMessage(m_hWnd, TB_REPLACEBITMAP, 0, (LPARAM)&replaceBitmap);
	}

	return TRUE;
}

BOOL CToolBar::SetButtons(const UINT *pIDArray, int nIDCount)
{

static TBBUTTON stbb[] = 
{
	{ 0, IDM_FILE_NEW,			TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 1, IDM_FILE_OPEN,			TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 2, IDM_FILE_SAVE,			TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 0, 0, 0, TBSTYLE_SEP, 0, 0},
	{ 3, IDM_EDIT_CUT,			TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 4, IDM_EDIT_COPY,			TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 5, IDM_EDIT_PASTE,		TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 0, 0, 0, TBSTYLE_SEP, 0, 0},
	{ 6, IDM_VIEW_HIERARCHY,	TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 },
	{ 0, 0, 0, TBSTYLE_SEP, 0, 0},
	{ 7, IDM_HELP_ABOUT,		TBSTATE_ENABLED, TBSTYLE_BUTTON, 0,0 }
};
return (BOOL) SendMessage(m_hWnd, TB_ADDBUTTONS, 10, (LPARAM)&stbb);

	// delete all existing buttons
	int nCount = (int)SendMessage(m_hWnd, TB_BUTTONCOUNT, 0, 0);
	while (nCount--)
		VERIFY(SendMessage(m_hWnd, TB_DELETEBUTTON, 0, 0));

	// pIDArray != NULL  --> not implemented see bartool.cpp of MFC
	ASSERT(pIDArray == NULL);

	{
		// add 'blank' buttons
		TBBUTTON button; memset(&button, 0, sizeof(TBBUTTON));
		button.fsState = TBSTATE_ENABLED;
		for(int i=0; i<nIDCount; i++)
		{
			ASSERT(button.fsStyle == TBSTYLE_BUTTON);
			if(!SendMessage(m_hWnd, TB_ADDBUTTONS, 1, (LPARAM)&button))
				return FALSE;
		}
	}

	Invalidate();

	return TRUE;
}

void CToolBar::SetButtonInfo(int nIndex, UINT nID, UINT nStyle, int iImage)
{
	TBBUTTON button;
return;
	button.iBitmap = iImage;
	button.idCommand = nID;
	button.fsState = TBSTATE_ENABLED;//(BYTE)HIWORD(nStyle);
	button.fsStyle = TBSTYLE_BUTTON;//(BYTE)LOWORD(nStyle);
	button.dwData = 0;
	button.iString = 0;

	VERIFY(SendMessage(m_hWnd, TB_DELETEBUTTON, nIndex, 0));
	VERIFY(SendMessage(m_hWnd, TB_INSERTBUTTON, nIndex, (LPARAM)&button));
}

// Positioning
int CToolBar::GetHeight() const
{
	return 24;
}

void CToolBar::SetWidth(int cx)
{
}

/////////////////////////////////////////////////////////////////////////////////////////////
