// For Win32
#ifndef __XFCWIN_H__
#define __XFCWIN_H__

typedef struct _XfcModuleState
{
	CWinApp* m_pCurrentWinApp;
	LPCTSTR m_pszAppName;

	HINSTANCE hInstance;
} XfcModuleState;

extern XfcModuleState xfcModuleState;

#define XfcGetHinstance()	(xfcModuleState.hInstance)

#define XfcGetModuleState()	(&xfcModuleState)
#define XfcGetApp()			(xfcModuleState.m_pCurrentWinApp)
#define XfcGetAppName()			(xfcModuleState.m_pszAppName)
#define XfcPostQuitMessage	PostQuitMessage

//////////////////////////////////////////////////////////////////////////////

class CMenu
{
public:
	CMenu();
	~CMenu();

	HMENU m_hMenu;

	CMenu* GetSubMenu(int nPos) const;

	BOOL AppendMenu(UINT nFlags, UINT nID =0, LPCTSTR pszLabel =NULL);
	UINT CheckMenuItem(UINT nIDCheckItem, UINT nCheck);
	UINT EnableMenuItem(UINT nIDEnableItem, UINT nEnable);
	BOOL ModifyMenu(UINT nPosition, UINT nFlags, UINT nID =0, LPCTSTR pszLabel =NULL);
	void TrackPopupMenu(UINT nFlags, int x, int y, CWnd* pWnd);
};

class CCmdTarget
{
//	DECLARE_DYNAMIC(CCmdTarget)
protected:

public:
// Constructors
	CCmdTarget();

//	XfcModuleState* m_pModuleState;

protected:
//	friend class CView;

//	CView* GetRoutingView();
//	CFrameWnd* GetRoutingFrame();
	DECLARE_MESSAGE_MAP()		// base class - no {{ }} macros
};

class CWnd : public CCmdTarget
{
public:
	HWND m_hWnd;

	CWnd();

	virtual BOOL DestroyWindow();

	// Window Operations
//	LRESULT SendMessage(UINT message, XEvent* pEvent);
	void SetWindowText(LPCTSTR pszString);
	int GetWindowText(LPTSTR pszStringBuf, int nMaxCount) const;
	void Invalidate(BOOL bErase =true);
	void UpdateWindow();
	BOOL ShowWindow(int nCmdShow);
	void MoveWindow(RECT* pRect, BOOL bRepaint =true);
	void MoveWindow(int x, int y, int nWidth, int nHeight,
		BOOL bRepaint =true);

	void ClientToScreen(POINT* pPoint) const;
	void ScreenToClient(POINT* pPoint) const;

	void SetCursor(HCURSOR hCursor);
	void SetCursorPos(int x, int y);
	void SetFocus();

	// Grab Mouse Pointer
	void SetCapture();
	BOOL ReleaseCapture();
	static CWnd* PASCAL GetCapture();

	inline HWND GetSafeHwnd() const { return m_hWnd; }
};

class CFrameWnd : public CWnd
{
public:
	CFrameWnd();

	void SetMenu(CMenu* pMenu);

protected:
	SIZE m_sizeFrameWnd;

	CStatusBar* m_pStatusBar;
	CToolBar* m_pToolBar;

	virtual void RecalcLayout(BOOL bNotify =true);
	virtual void OnSizeClient(RECT *pRectClient);

	afx_msg void OnClose();
	afx_msg void OnSize(int cx, int cy);

	friend class CToolBar;
	friend class CStatusBar;
};

class CControlBar : public CWnd
{
public:
	CControlBar() { m_bHidden = false; }

	bool ToggleShow();

	inline bool IsHidden() { return m_bHidden; }

private:
	bool m_bHidden;
};

class CToolBar : public CControlBar
{
public:
	CToolBar();
	~CToolBar();

	BOOL Create(CFrameWnd* pParentWnd, DWORD dwStyle =WS_CHILD | WS_VISIBLE,
		UINT nID=0);
	BOOL LoadBitmap(UINT nResID);
	BOOL SetButtons(const UINT *pIDArray, int nIDCount);
	void SetButtonInfo(int nIndex, UINT nID, UINT nStyle, int iImage);

// Positioning
	int GetHeight() const;
	void SetWidth(int cx);

};

class CStbPaneInfo
{
public:
	int cxWidth;
};

class CStatusBar : public CControlBar
{
public:
	CStatusBar();
	~CStatusBar();

	BOOL Create(CFrameWnd* pParentWnd, DWORD dwStyle =WS_VISIBLE, UINT nID=0);
	BOOL SetIndicators(const UINT *pIDArray, int nIDCount);

	void SetPaneInfo(int nIndex, UINT nID, UINT nStyle, int cxWidth);
	BOOL SetPaneText(int nIndex, LPCTSTR pszNewText, BOOL bUpdate =TRUE);
	void SetPaneStyle(int nIndex, UINT nStyle);

// Positioning
	int GetHeight() const;
	void SetWidth(int cx);

	int *m_pRightEdges;

protected:
	int m_nPaneCount;
	CStbPaneInfo* m_pPaneInfos;
};

class CWinApp
{
public:
	CWinApp(LPCTSTR pszAppName =NULL);
	~CWinApp();

	CWnd* m_pMainWnd;

	virtual BOOL InitApplication();
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	virtual int Run();

private:
	MSG* m_pMsgCur;
};

int XfcMessageBox(LPCTSTR pszText, UINT nType =MB_OK, UINT nIDHelp =0);
int XfcMessageBox(UINT nIDPrompt, UINT nType =MB_OK, UINT nIDHelp =(UINT)-1);

#endif