#include "xfc.h"

void CDC::SetFgColor(UINT nColor)
{
	XSetForeground(XfcGetDisplay(), m_hGC, nColor);
}

void CDC::SetBkColor(UINT nColor)
{
	XSetBackground(XfcGetDisplay(), m_hGC, nColor);
}

void CDC::SetFont(HFONT hFont)
{
	XSetFont(XfcGetDisplay(), m_hGC, hFont);
}

void CDC::DrawPoint(int x, int y)
{
	XDrawPoint(XfcGetDisplay(), m_hDrawable, m_hGC, x, y);
}

void CDC::DrawLine(int x1, int y1, int x2, int y2)
{
	XDrawLine(XfcGetDisplay(), m_hDrawable, m_hGC, x1, y1, x2, y2);
}

void CDC::Rectangle(int x1, int y1, int x2, int y2)
{
	XDrawRectangle(XfcGetDisplay(), m_hDrawable, m_hGC,
		x1, y1, x2 - x1, y2 - y1);
}

void CDC::FillRect(short x, short y, WORD width, WORD height)
{
	XFillRectangle(XfcGetDisplay(), m_hDrawable, m_hGC,
		x, y, width, height);
}

void CDC::DrawString(int x, int y, LPCTSTR pszString)
{
	XDrawString(XfcGetDisplay(), m_hDrawable, m_hGC, x, y,
		pszString, strlen(pszString));
}

//////////////////////////////////////////////////////////////////////////////

CPaintDC::CPaintDC(CWnd* pWnd)
{
	XGCValues values;
	Display* pDisplay = XfcGetDisplay();

	m_hDrawable = pWnd->GetSafeHwnd();
	m_hGC = XCreateGC(pDisplay, m_hDrawable, 0, &values);
	XSetGraphicsExposures(pDisplay, m_hGC, FALSE);
}

CPaintDC::~CPaintDC()
{
	XFreeGC(XfcGetDisplay(), m_hGC);
	m_hDrawable = NULL;
}

//////////////////////////////////////////////////////////////////////////////

CClientDC::CClientDC(CWnd* pWnd)
{
	XGCValues values;
	Display* pDisplay = XfcGetDisplay();

	m_hDrawable = pWnd->GetSafeHwnd();
	m_hGC = XCreateGC(pDisplay, m_hDrawable, 0, &values);
	XSetGraphicsExposures(pDisplay, m_hGC, FALSE);
}

CClientDC::~CClientDC()
{
	XFreeGC(XfcGetDisplay(), m_hGC);
	m_hDrawable = NULL;
}

//////////////////////////////////////////////////////////////////////////////

bool CRect::PtInRect(int x, int y) const
{
	return (left < x && x < right && top < y && y < bottom) ? true : false;
}

bool CRect::PtInRect(POINT point) const
{
	return (left < point.x && point.x < right
	&& top < point.y && point.y < bottom) ? true : false;
}

bool CRect::EqualRect(RECT* pRect) const
{
	return (left == pRect->left && right == pRect->right
	&& top == pRect->top && bottom == pRect->bottom) ? true : false;
}

void CRect::CopyRect(RECT* pSrcRect)
{
	top = pSrcRect->top;
	left = pSrcRect->left;
	right = pSrcRect->right;
	bottom = pSrcRect->bottom;
}

void CRect::SetRect(int x1, int y1, int x2, int y2)
{
	left = x1;
	top = y1;
	right = x2;
	bottom = y2;
}

void CRect::InflateRect(int x, int y)
{
	left -= x;
	top -= y;
	right += x;
	bottom += y;
}
