#include "xfc.h"

//////////////////////////////////////////////////////////////////////////////
// CFrameWnd

#define __PMF CFrameWnd
BEGIN_MESSAGE_MAP(CFrameWnd, CWnd)
	ON_SIZE()
	ON_CLOSE()
END_MESSAGE_MAP()
#undef __PMF

CFrameWnd::CFrameWnd()
{
	m_pMenuBar = NULL;
	m_pStatusBar = NULL;
	m_pToolBar = NULL;
}

CFrameWnd::~CFrameWnd()
{
	if(m_pMenuBar != NULL) delete m_pMenuBar;
}

bool CFrameWnd::Create(LPCTSTR pszWindowName,
	DWORD dwStyle, const RECT* pRect, CWnd* pParentWnd, void* pCreateParam)
{
	static const RECT rectDefault = { 0,0,320,240 };

	if(pRect == NULL) pRect = &rectDefault;

	if(!CWnd::Create(pszWindowName, dwStyle,
		pRect->left, pRect->top,
		pRect->right - pRect->left, pRect->bottom - pRect->top,
		pParentWnd, pCreateParam))
	{
		TRACE0("Warning: failed to create CFrameWnd.\n");
		return false;
	}

	return true;
}

void CFrameWnd::SetMenu(CMenu* pMenu)
{
	if(m_pMenuBar != NULL)
	{
		delete m_pMenuBar;
		m_pMenuBar = NULL;
	}

	if(pMenu)
	{
		m_pMenuBar = new CMenuBar();
		m_pMenuBar->Create(pMenu, this);
		m_pMenuBar->ShowWindow(SW_SHOW);
	}
}

//////////////////////////////////////////////////////////////////////////////

void CFrameWnd::OnSizeClient(RECT* pRectClient)
{
}

//////////////////////////////////////////////////////////////////////////////

void CFrameWnd::OnSize(int cx, int cy)
{
	m_sizeFrameWnd.cx = cx;
	m_sizeFrameWnd.cy = cy;

	RecalcLayout();
}

void CFrameWnd::RecalcLayout(bool bNotify)
{
	RECT rcClient;

	rcClient.left = rcClient.top = 0;
	rcClient.right = m_sizeFrameWnd.cx;
	rcClient.bottom = m_sizeFrameWnd.cy;

	if(m_pMenuBar != NULL)
	{
		rcClient.top += m_pMenuBar->GetHeight();
		m_pMenuBar->MoveWindow(0,0, m_sizeFrameWnd.cx,
			m_pMenuBar->GetHeight());
		m_pMenuBar->SetWidth(m_sizeFrameWnd.cx);
	}

	if(m_pToolBar != NULL)
	{
		m_pToolBar->MoveWindow(0, rcClient.top, m_sizeFrameWnd.cx,
			m_pToolBar->GetHeight());
		rcClient.top += m_pToolBar->GetHeight();
		m_pToolBar->SetWidth(m_sizeFrameWnd.cx);
	}

	if(m_pStatusBar != NULL)
	{
		rcClient.bottom -= m_pStatusBar->GetHeight();
		m_pStatusBar->MoveWindow(0, rcClient.bottom,
			m_sizeFrameWnd.cx, m_pStatusBar->GetHeight());
		m_pStatusBar->SetWidth(m_sizeFrameWnd.cx);
	}

	OnSizeClient(&rcClient);
}

void CFrameWnd::OnClose()
{
	CWinApp* pApp = XfcGetApp();
	if(pApp->m_pMainWnd == this)
	{
		TRACE("CFrameWnd::OnClose() -- MainFrame close\n");
		XfcPostQuitMessage(0);
	}

	SetMenu(NULL);
	DestroyWindow();
}
