#include "xfc.h"

//////////////////////////////////////////////////////////////////////////////
// CMenu - without any GUI action

CMenu::CMenu()
{
}

CMenu::~CMenu()
{
	POSITION pos = m_listMenu.GetHeadPosition();
	while(pos)
	{
		CMenuItem* pItem = m_listMenu.GetNext(pos);
		delete [] pItem->pszLabel;
		delete pItem;
	}

	m_listMenu.RemoveAll();
}

CMenu* CMenu::GetSubMenu(int nPos) const
{
	POSITION pos = m_listMenu.GetHeadPosition();

	while(pos)
	{
		CMenuItem *p = m_listMenu.GetNext(pos);
		if(nPos-- == 0)
		{
			if(p->nFlags & MF_POPUP)
				return p->data.pSubMenu;
			return NULL;
		}
	}

	return NULL;
}

bool CMenu::AppendMenu(UINT nFlags, UINT nID, LPCTSTR pszLabel)
{
	char *p;
	CMenuItem *pItem = new CMenuItem;

	pItem->nFlags = nFlags;
	
	if(nFlags == MF_SEPARATOR) goto AddNow;

	pItem->nStatus = 0;

	ASSERT(pszLabel != NULL);
	pItem->pszLabel = new TCHAR[strlen(pszLabel) +1];
	strcpy(pItem->pszLabel, pszLabel);
	if((p = strchr(pItem->pszLabel, '\t')) != NULL)
	{
		*p++ = '\0';
		pItem->pszShortcut = p;
		pItem->nShortcutLen = strlen(pItem->pszShortcut);

		pszLabel = pItem->pszLabel;
	}
	else
	{
		pItem->pszShortcut = NULL;
		pItem->nShortcutLen = 0;
	}

	pItem->nLabelLen = strlen(pszLabel);
	if((p = strchr(pszLabel, '&')) != NULL)
	{
		pItem->nHotPos = p - pszLabel;
		strncpy(&pItem->pszLabel[pItem->nHotPos],
			&pItem->pszLabel[pItem->nHotPos +1],
			pItem->nLabelLen - pItem->nHotPos);
	}
	else
	{
		pItem->nHotPos = -1;
	}

	if(nFlags & MF_POPUP)
		pItem->data.pSubMenu = (CMenu*) nID;
	else if(nFlags & MF_STRING)
		pItem->data.nID = nID;

AddNow:
	m_listMenu.AddTail(pItem);

	return true;
}

UINT CMenu::CheckMenuItem(UINT nIDCheckItem, UINT nCheck)
{
	POSITION pos;
	UINT nRet;

	pos = m_listMenu.GetHeadPosition();
	if(nCheck & MF_BYPOSITION)
	{
		while(pos)
		{
			CMenuItem *p = m_listMenu.GetNext(pos);
			if(nIDCheckItem-- == 0)
			{
				if(p->nFlags & MF_POPUP) return UINT(-1);

				nRet = p->nFlags & MF_CHECKED;

				if(nCheck & MF_CHECKED)
					p->nFlags |= MF_CHECKED;
				else
					p->nFlags &= (-1 ^ MF_CHECKED);

				return nRet;
			}
		}
	}
	else
	{
		// default: MF_BYCOMMAND
		while(pos)
		{
			CMenuItem *p = m_listMenu.GetNext(pos);
			if(p->nFlags & MF_POPUP)
			{
				nRet = p->data.pSubMenu->CheckMenuItem(nIDCheckItem, nCheck);
				if(nRet != UINT(-1)) return nRet;
			}
			else if(p->data.nID == nIDCheckItem)
			{
				nRet = p->nFlags & MF_CHECKED;

				if(nCheck & MF_CHECKED)
					p->nFlags |= MF_CHECKED;
				else
					p->nFlags &= (-1 ^ MF_CHECKED);

				return nRet;
			}
		}
	}

	return UINT(-1);
}

UINT CMenu::EnableMenuItem(UINT nIDEnableItem, UINT nEnable)
{
	POSITION pos;
	UINT nRet;

	pos = m_listMenu.GetHeadPosition();
	if(nEnable & MF_BYPOSITION)
	{
		while(pos)
		{
			CMenuItem *p = m_listMenu.GetNext(pos);
			if(nIDEnableItem-- == 0)
			{
				nRet = p->nFlags & (MF_ENABLED | MF_DISABLED | MF_GRAYED);
				p->nFlags &= (-1 ^ (MF_ENABLED | MF_DISABLED | MF_GRAYED));
				p->nFlags |= nEnable & (MF_ENABLED | MF_DISABLED | MF_GRAYED);
				return nRet;
			}
		}
	}
	else
	{
		// default: MF_BYCOMMAND
		while(pos)
		{
			CMenuItem *p = m_listMenu.GetNext(pos);
			if(p->nFlags & MF_POPUP)
			{
				nRet = p->data.pSubMenu->EnableMenuItem(nIDEnableItem, nEnable);
				if(nRet != UINT(-1)) return nRet;
			}
			else if(p->data.nID == nIDEnableItem)
			{

				nRet = p->nFlags & (MF_ENABLED | MF_DISABLED | MF_GRAYED);
				p->nFlags &= (-1 ^ (MF_ENABLED | MF_DISABLED | MF_GRAYED));
				p->nFlags |= nEnable & (MF_ENABLED | MF_DISABLED | MF_GRAYED);
				return nRet;
			}
		}
	}
	
	return UINT(-1);
}

bool CMenu::ModifyMenu(UINT nPosition, UINT nFlags, UINT nID, LPCTSTR pszLabel)
{
TRACE("ModifyMenu(%d, %s)\n", nID, pszLabel);
	return false;
}

void CMenu::TrackPopupMenu(UINT nFlags, int x, int y, CWnd* pWnd)
{
	CPopupMenu* pPopupMenu;

	pPopupMenu = new CPopupMenu();
	pPopupMenu->Create(pWnd, NULL, true);
	pPopupMenu->Show(x, y, this);

	XfcGetApp()->SetMenuTracker(new CMenuTracker(NULL, pPopupMenu));
}

//////////////////////////////////////////////////////////////////////////////

#define __PMF CMenuBar
BEGIN_MESSAGE_MAP(CMenuBar, CWnd)
	ON_MOUSEMOVE()
	ON_LBUTTONDOWN()
	ON_LBUTTONUP()
	ON_DESTROY()
	ON_DRAW()
END_MESSAGE_MAP()
#undef __PMF

CMenuBar::~CMenuBar()
{
	if(m_pMenubarItems) delete [] m_pMenubarItems;
	DestroyWindow();
}

bool CMenuBar::Create(CMenu* pMenu, CWnd* pParentWnd)
{
	m_pMenubarItems = NULL;
	m_nMenubarItems = 0;

	m_pParentWnd = pParentWnd;
	m_pMenuItemList = &pMenu->m_listMenu;

	RebuildLayout();

//	m_nBarWidth = 100;

	if(CWnd::Create("", 0,
		0,0,100, 32,	// default size
		pParentWnd, NULL))
	{
		XGrabButton(XfcGetDisplay(), AnyButton,
			AnyModifier, m_hWnd, True,
			ButtonPressMask | ButtonReleaseMask | EnterWindowMask,
			GrabModeAsync, GrabModeAsync, None, None);
		return true;
	}

	return false;
}

void CMenuBar::OnDestroy()
{
	TRACE("CMenuBar::OnDestroy()\n");
}

int CMenuBar::GetHeight() const
{
	return m_nBarHeight;
}

void CMenuBar::SetWidth(int cx)
{
	m_nBarWidth = cx;
}

void CMenuBar::RebuildLayout()
{
#define MENUBAR_MARGIN_X	4
	int n;

	m_pSelectedItem = NULL;
	m_bUnderTracking = false;

	n = m_pMenuItemList->GetCount();
	ASSERT(n > 0);
	if(n != m_nMenubarItems)
	{
		if(m_pMenubarItems) delete [] m_pMenubarItems;
		m_pMenubarItems = new struct MenubarItem[n];
		m_nMenubarItems = n;
	}

	n = 0;
	short x = 0;
	short cy = CFont::s_pFontStruct_Normal->ascent
		+ CFont::s_pFontStruct_Normal->descent +2;
	POSITION pos = m_pMenuItemList->GetHeadPosition();
	while(pos)
	{
		CMenuItem* pItem = m_pMenuItemList->GetNext(pos);

		m_pMenubarItems[n].left = x;
		m_pMenubarItems[n].right = x + XTextWidth(CFont::s_pFontStruct_Normal, pItem->pszLabel, pItem->nLabelLen) + MENUBAR_MARGIN_X*2;
		m_pMenubarItems[n].top = 0;
		m_pMenubarItems[n].bottom = cy;
		m_pMenubarItems[n].pItem = pItem;

		x = m_pMenubarItems[n].right;
		n++;
	}

	m_nBarHeight = cy +2;
}

void CMenuBar::DrawItem(CDC* pDC, struct MenubarItem* pBarItem, bool bSelect)
{
	int cyFont = CFont::s_pFontStruct_Normal->ascent +1;
	pDC->SetFont(CFont::s_hFont_Normal);

	pDC->SetFgColor(bSelect ? CColor::darkBlue : CColor::lightGray);
	pDC->FillRect(pBarItem->left, pBarItem->top,
		pBarItem->right - pBarItem->left,
		pBarItem->bottom - pBarItem->top);
	pDC->SetFgColor(bSelect ? CColor::white : CColor::black);

	int x = pBarItem->left + MENUBAR_MARGIN_X;
	int y = pBarItem->top + cyFont;

	CMenuItem* pItem = pBarItem->pItem;
	pDC->DrawString(x, y, pItem->pszLabel);

	if(pItem->nHotPos >= 0)
	{
		pDC->DrawLine(
		x + XTextWidth(CFont::s_pFontStruct_Normal, pItem->pszLabel, pItem->nHotPos), y+1,
		x + XTextWidth(CFont::s_pFontStruct_Normal, pItem->pszLabel, pItem->nHotPos +1) -1, y +1);
	}
}

struct CMenuBar::MenubarItem* CMenuBar::GetBarItemByPos(int x, int y)
{
	if(y > 0 && y < m_nBarHeight)
	{
		struct MenubarItem* pItem = &m_pMenubarItems[0];
		int n = m_nMenubarItems;

		while(n-- > 0)
		{
			if(x < pItem->right) return pItem;
			pItem++;
		}
	}
	
	return NULL;
}

void CMenuBar::ReleaseTracking()
{
	ASSERT(m_bUnderTracking == true);

	// destroy already popuped menu
	if(m_pPopupMenu != NULL)
	{
		delete m_pPopupMenu;
		m_pPopupMenu = NULL;
	}

	// erase previous invert
	if(m_pSelectedItem != NULL)
	{
		CClientDC dc(this);
		DrawItem(&dc, m_pSelectedItem, false);
		m_pSelectedItem = NULL;
	}

	ReleaseCapture();
	m_bUnderTracking = false;
}

void CMenuBar::OnLButtonDown(UINT nFlags, int x, int y)
{
	struct MenubarItem *pBarItem;

TRACE("CMenuBar::OnLButtonDown(%x, %d, %d)\n", nFlags, x, y);

	pBarItem = GetBarItemByPos(x, y);

	if(pBarItem == NULL)
	{
		if(m_bUnderTracking)
		{
			// Release Tracking
			XfcGetApp()->SetMenuTracker(NULL);
		}
		return;
	}

	if(m_bUnderTracking == true)
	{
		XfcGetApp()->SetMenuTracker(NULL);
		return;
	}

	// Start Tracking

	m_bUnderTracking = true;
	m_pSelectedItem = pBarItem;
	SetCapture();

	if(m_pSelectedItem->pItem->nFlags & MF_POPUP)
	{
		POINT pt = { m_pSelectedItem->left, m_pSelectedItem->bottom };

		ASSERT(m_pPopupMenu == NULL);
		m_pPopupMenu = new CPopupMenu();
		m_pPopupMenu->Create(m_pParentWnd, this);
		ClientToScreen(&pt);
		m_pPopupMenu->Show(pt.x, pt.y,
			m_pSelectedItem->pItem->data.pSubMenu);
	}

	// draw current invert
	CClientDC dc(this);
	DrawItem(&dc, m_pSelectedItem, true);

	CMenuTracker* pTracker = new CMenuTracker(this, NULL);
	XfcGetApp()->SetMenuTracker(pTracker);
}

void CMenuBar::OnLButtonUp(UINT nFlags, int x, int y)
{
	struct MenubarItem *pBarItem;

TRACE("CMenuBar::OnLButtonUp(%x, %d, %d)\n", nFlags, x, y);

	pBarItem = GetBarItemByPos(x, y);

	if(pBarItem == NULL)
	{
		if(m_bUnderTracking)
			XfcGetApp()->SetMenuTracker(NULL);
		return;
	}

	CMenuItem* pItem = pBarItem->pItem;
	if(pItem->nFlags & MF_STRING)
	{
		XFCMSG msg;

		msg.window = m_pParentWnd->GetSafeHwnd();
		msg.message = XWM_COMMAND;
		msg.xevent.pad[0] = pItem->data.nID;
		msg.xevent.pad[1] = 0;

		XfcPostMessage(&msg);
		XfcGetApp()->SetMenuTracker(NULL);
	}
}

void CMenuBar::OnMouseMove(UINT nFlags, int x, int y)
{
	struct MenubarItem *pBarItem;

	if(m_bUnderTracking == false) return;

	pBarItem = GetBarItemByPos(x, y);

	if(pBarItem == NULL) return;
	if(pBarItem == m_pSelectedItem) return;

	CClientDC dc(this);
	// restore previous invert
	if(m_pSelectedItem != NULL)
		DrawItem(&dc, m_pSelectedItem, false);

	m_pSelectedItem = pBarItem;

	CMenuItem* pItem = pBarItem->pItem;
	if(pItem->nFlags & MF_POPUP)
	{
		ASSERT(m_pPopupMenu != NULL);

		POINT pt;

		pt.x = pBarItem->left;
		pt.y = pBarItem->bottom;
		ClientToScreen(&pt);
		m_pPopupMenu->Show(pt.x, pt.y, pItem->data.pSubMenu);

		DrawItem(&dc, m_pSelectedItem, true);
	}
}

void CMenuBar::OnDraw(CDC* pDC)
{
	pDC->SetFgColor(CColor::lightGray);
	pDC->FillRect(0,0, m_nBarWidth, m_nBarHeight);

	for(int n=0; n<m_nMenubarItems; n++)
	{
		DrawItem(pDC, &m_pMenubarItems[n],
			&m_pMenubarItems[n] == m_pSelectedItem);
	}
}

//////////////////////////////////////////////////////////////////////////////

#define __PMF CPopupMenu
BEGIN_MESSAGE_MAP(CPopupMenu, CWnd)
	ON_DRAW()
	ON_MOUSEMOVE()
	ON_LBUTTONDOWN()
	ON_LBUTTONUP()
	ON_RBUTTONDOWN()
	ON_RBUTTONUP()
	ON_DESTROY()
	ON_LEAVEWINDOW()
END_MESSAGE_MAP()
#undef __PMF

#define POPUPMENU_LABEL_XPOS		16
#define POPUPMENU_SHORTCUT_MARGIN	6
#define POPUPMENU_SUBPOPUP_WIDTH	12

CPopupMenu::CPopupMenu()
{
	m_pSubPopupMenu = NULL;
	m_bStandAlone = false;
}

CPopupMenu::~CPopupMenu()
{
	if(m_bStandAlone)
	{
/*
		ReleaseCapture();
		XUngrabButton(XfcGetDisplay(), AnyButton, AnyModifier, m_hWnd);
*/
	}

	if(m_pSubPopupMenu) delete m_pSubPopupMenu;
	DestroyWindow();
}

bool CPopupMenu::Create(CWnd* pNotifyWnd, CWnd* pParentWnd, bool bStandAlone)
{
	m_pNotifyWnd = pNotifyWnd;
	m_pParentWnd = pParentWnd;
	m_pPopupmenuItems = NULL;
	m_nPopupmenuItems = 0;
	m_pMenuItemList = NULL;
	m_bHidden = true;

	ASSERT(m_pNotifyWnd != NULL);

	m_nItemHeight = CFont::s_pFontStruct_Normal->ascent
		+ CFont::s_pFontStruct_Normal->descent +3;
	m_nFontYAdjust = CFont::s_pFontStruct_Normal->ascent +1;

	if(CWnd::Create("", WS_POPUP|WS_SYSMENU,
		10,10,32,32, NULL, NULL) == false) return false;

	if(bStandAlone)
	{
		m_bStandAlone = bStandAlone;
/*
		XGrabButton(XfcGetDisplay(), AnyButton,
			AnyModifier, m_hWnd, True,
			ButtonPressMask | ButtonReleaseMask | EnterWindowMask,
			GrabModeAsync, GrabModeAsync, None, None);

		SetCapture();
*/
	}

	return true;
}

void CPopupMenu::OnDestroy()
{
	TRACE("CPopupMenu::OnDestroy(me:%X)\n", m_hWnd);
}

void CPopupMenu::Show(int x, int y, CMenu* pMenu)
{
	if(m_pMenuItemList != &pMenu->m_listMenu)
	{
		m_pMenuItemList = &pMenu->m_listMenu;
		RebuildLayout();
		MoveWindow(x, y, m_nBoxWidth +1, m_nBoxHeight +1);
	}

	if(m_bHidden)
	{
		ShowWindow(SW_SHOW);
		m_bHidden = false;
	}
}

void CPopupMenu::Hide()
{
	ASSERT(m_bStandAlone == false);

	if(m_bHidden == false)
	{
		if(m_pSubPopupMenu != NULL) m_pSubPopupMenu->Hide();

		m_pSelectedItem = NULL;

		ShowWindow(SW_HIDE);
		m_bHidden = true;
	}
}

void CPopupMenu::RebuildLayout()
{
	int n;
	int cxLabel, cxShortcut;
	int cxMaxLabel =0, cxMaxShortcut =0;

	m_pSelectedItem = NULL;

	n = m_pMenuItemList->GetCount();
	ASSERT(n > 0);
	if(n != m_nPopupmenuItems)
	{
		if(m_pPopupmenuItems) delete [] m_pPopupmenuItems;
		m_pPopupmenuItems = new struct PopupmenuItem[n];
		m_nPopupmenuItems = n;
	}

	n = 0;
	short y = 3;
	POSITION pos = m_pMenuItemList->GetHeadPosition();
	while(pos)
	{
		CMenuItem *pItem = m_pMenuItemList->GetNext(pos);

		m_pPopupmenuItems[n].pItem = pItem;
		m_pPopupmenuItems[n].top = y;

		if(pItem->nFlags & MF_SEPARATOR)
		{
			m_pPopupmenuItems[n].bottom = y +8;
		}
		else
		{
			m_pPopupmenuItems[n].bottom = y + m_nItemHeight;

			cxLabel = XTextWidth(CFont::s_pFontStruct_Normal,
				pItem->pszLabel, pItem->nLabelLen);
			cxShortcut = (pItem->pszShortcut != NULL) ?
				XTextWidth(CFont::s_pFontStruct_Normal,
				pItem->pszShortcut, pItem->nShortcutLen) : 0;

			if(cxMaxLabel < cxLabel) cxMaxLabel = cxLabel;
			if(cxMaxShortcut < cxShortcut) cxMaxShortcut = cxShortcut;

			if(pItem->nFlags & MF_POPUP && m_pSubPopupMenu == NULL)
			{
				m_pSubPopupMenu = new CPopupMenu();
				m_pSubPopupMenu->Create(m_pNotifyWnd, this);
			}
		}

		y = m_pPopupmenuItems[n].bottom;
		n++;
	}

	m_nBoxHeight = y + 3;
	m_nShortcutXPos = cxMaxLabel
		+ (POPUPMENU_LABEL_XPOS + POPUPMENU_SHORTCUT_MARGIN);
	m_nBoxWidth = (cxMaxLabel + cxMaxShortcut)
		+ (POPUPMENU_LABEL_XPOS + POPUPMENU_SHORTCUT_MARGIN
		+ POPUPMENU_SUBPOPUP_WIDTH);
	m_nItemRectWidth = m_nBoxWidth -5;
}

struct CPopupMenu::PopupmenuItem *CPopupMenu::GetPmItemByPos(int x, int y)
{
	if(x > 3 && y > 3 && x < m_nBoxWidth && y < m_nBoxHeight)
	{
		struct PopupmenuItem* pItem = &m_pPopupmenuItems[0];
		int n = m_nPopupmenuItems;

		while(n-- > 0)
		{
			if(y < pItem->bottom) return pItem;
			pItem++;
		}
	}
	
	return NULL;
}

void CPopupMenu::OnLeaveWindow()
{
	if(m_pSelectedItem != NULL)
	{
		CClientDC dc(this);

		DrawItem(&dc, m_pSelectedItem, false);
		m_pSelectedItem = NULL;
	}
}

void CPopupMenu::OnMouseMove(UINT nFlags, int x, int y)
{
	struct PopupmenuItem *pPmItem;

	pPmItem = GetPmItemByPos(x, y);

	if(pPmItem == NULL) return;
	if(pPmItem == m_pSelectedItem) return;


	CClientDC dc(this);

	if(m_pSelectedItem != NULL)
	{
		// erase previous invert
		DrawItem(&dc, m_pSelectedItem, false);
	}

	CMenuItem* pItem = pPmItem->pItem;
	if(pItem->nFlags == MF_SEPARATOR)
	{
		m_pSelectedItem = NULL;
		if(m_pSubPopupMenu != NULL) m_pSubPopupMenu->Hide();

		return;
	}

	m_pSelectedItem = pPmItem;

	if(pItem->nFlags & MF_POPUP)
	{
		ASSERT(m_pSubPopupMenu != NULL);

		POINT pt;

		pt.x = m_nBoxWidth - 4;
		pt.y = pPmItem->top;
		ClientToScreen(&pt);
		m_pSubPopupMenu->Show(pt.x, pt.y, pItem->data.pSubMenu);
	}
	else
	{
		if(m_pSubPopupMenu != NULL) m_pSubPopupMenu->Hide();
	}

	// invert current
	DrawItem(&dc, pPmItem, true);
}

void CPopupMenu::OnRButtonDown(UINT nFlags, int x, int y)
{ TRACE("R-"); OnLButtonDown(nFlags, x, y); }
void CPopupMenu::OnLButtonDown(UINT nFlags, int x, int y)
{
	TRACE("CPopupMenu::OnLButtonDown(%x, %d, %d)\n", nFlags, x, y);
}

void CPopupMenu::OnRButtonUp(UINT nFlags, int x, int y)
{ TRACE("R-"); OnLButtonUp(nFlags, x, y); }
void CPopupMenu::OnLButtonUp(UINT nFlags, int x, int y)
{
	TRACE("CPopupMenu::OnLButtonUp(%x, %d, %d)\n", nFlags, x, y);

	struct PopupmenuItem *pPmItem = GetPmItemByPos(x, y);

	if(pPmItem != NULL)
	{
		CMenuItem* pItem = pPmItem->pItem;

		if(pItem->nFlags & MF_STRING)
		{
			XFCMSG msg;
			msg.window = m_pNotifyWnd->GetSafeHwnd();
			msg.message = XWM_COMMAND;
			msg.xevent.pad[0] = pItem->data.nID;
			msg.xevent.pad[1] = 0;

			XfcPostMessage(&msg);
			XfcGetApp()->SetMenuTracker(NULL);
		}
	}
}

static XPoint ptCheck[3] = { {0,0}, {2,2}, {4,-4} };
static XPoint ptPopup[3] = { {0,0}, {0,-8}, {4,4} };

void CPopupMenu::DrawItemText(CDC* pDC, int x, int y, CMenuItem* pItem, XFontStruct *pFS)
{
	XDrawString(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
		x + POPUPMENU_LABEL_XPOS, y,
		pItem->pszLabel, pItem->nLabelLen);

	if(pItem->nHotPos >= 0)
	{
		XDrawLine(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			x + POPUPMENU_LABEL_XPOS + XTextWidth(pFS, pItem->pszLabel, pItem->nHotPos),
			y +1,
			x + POPUPMENU_LABEL_XPOS + XTextWidth(pFS, pItem->pszLabel, pItem->nHotPos +1) -1,
			y +1);
	}

	if(pItem->pszShortcut != NULL)
	{
		XDrawString(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			x + m_nShortcutXPos, y,
			pItem->pszShortcut, pItem->nShortcutLen);
	}

	if(pItem->nFlags & MF_CHECKED)
	{
		// Draw Check Mark
		ptCheck[0].x = x + 5;
		ptCheck[0].y = y - 6;
		XDrawLines(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			ptCheck, 3, CoordModePrevious);
		ptCheck[0].y++;
		XDrawLines(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			ptCheck, 3, CoordModePrevious);
		ptCheck[0].y++;
		XDrawLines(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			ptCheck, 3, CoordModePrevious);
	}
	else if(pItem->nFlags & MF_POPUP)
	{
		// Draw Triangle Pattern
		ptPopup[0].x = x + m_nBoxWidth - POPUPMENU_SUBPOPUP_WIDTH;
		ptPopup[0].y = y;
		XFillPolygon(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			ptPopup, 3, Convex, CoordModePrevious);
	}
}

void CPopupMenu::DrawItem(CDC* pDC, struct PopupmenuItem* pPmItem, bool bSelect)
{
	int y = pPmItem->top + m_nFontYAdjust;
	CMenuItem* pItem = pPmItem->pItem;

	XFontStruct* pFS = CFont::s_pFontStruct_Normal;
	pDC->SetFont(CFont::s_hFont_Normal);

	pDC->SetFgColor(bSelect ? CColor::darkBlue : CColor::lightGray);
	pDC->FillRect(3, pPmItem->top,
		m_nItemRectWidth, pPmItem->bottom - pPmItem->top +1);

	if(pItem->nFlags & MF_GRAYED)
	{
		if(bSelect)
		{
			pDC->SetFgColor(CColor::gray);
			DrawItemText(pDC, 0, y, pItem, pFS);
		}
		else
		{
			pDC->SetFgColor(CColor::white);
			DrawItemText(pDC, 0, y, pItem, pFS);
			pDC->SetFgColor(CColor::darkGray);
			DrawItemText(pDC, -1, y-1, pItem, pFS);
		}
	}
	else
	{
		pDC->SetFgColor(bSelect ? CColor::white : CColor::black);
		DrawItemText(pDC, 0, y, pItem, pFS);
	}
}

void CPopupMenu::OnDraw(CDC* pDC)
{
	pDC->SetFgColor(CColor::lightGray);
	pDC->FillRect(0,0, m_nBoxWidth, m_nBoxHeight);

	pDC->SetFgColor(CColor::white);
	pDC->DrawLine(1, 1, m_nBoxWidth, 1);
	pDC->DrawLine(1, 1, 1, m_nBoxHeight);

	pDC->SetFgColor(CColor::darkGray);
	pDC->DrawLine(1, m_nBoxHeight-1, m_nBoxWidth, m_nBoxHeight-1);
	pDC->DrawLine(m_nBoxWidth-1, 1, m_nBoxWidth-1, m_nBoxHeight);

	pDC->SetFgColor(CColor::black);
	pDC->DrawLine(m_nBoxWidth, 0, m_nBoxWidth, m_nBoxHeight);
	pDC->DrawLine(0, m_nBoxHeight, m_nBoxWidth, m_nBoxHeight);

	for(int n=0; n<m_nPopupmenuItems; n++)
	{

		if(m_pPopupmenuItems[n].pItem->nFlags == MF_SEPARATOR)
		{
			pDC->SetFgColor(CColor::darkGray);
			int y = m_pPopupmenuItems[n].top +3;
			pDC->DrawLine(3, y, m_nItemRectWidth +2, y);
			pDC->SetFgColor(CColor::white);
			y++; pDC->DrawLine(3, y, m_nItemRectWidth +2, y);
			pDC->SetFgColor(CColor::black);

			continue;
		}
		else
			DrawItem(pDC, &m_pPopupmenuItems[n],
				&m_pPopupmenuItems[n] == m_pSelectedItem);
	}
}

//////////////////////////////////////////////////////////////////////////////
// CMenuTracker

CMenuTracker::CMenuTracker(CMenuBar* pMB, CPopupMenu* pPM)
{
	m_pMenuBar = pMB;
	m_pPopupMenu = pPM;

	ASSERT( (pMB == NULL && pPM != NULL) ||
		(pMB != NULL && pPM == NULL));
}

CMenuTracker::~CMenuTracker()
{
	if(m_pMenuBar != NULL)
	{
		m_pMenuBar->ReleaseTracking();
	}
	else
	{
		// delete popupmenu on the fly
		delete m_pPopupMenu;
	}
}

bool CMenuTracker::IsOurMessage(HWND hWnd)
{
	CPopupMenu* p;

	if(m_pMenuBar != NULL)
	{
		if(hWnd == m_pMenuBar->m_hWnd) return true;
		p = m_pMenuBar->m_pPopupMenu;
	}
	else
	{
		p = m_pPopupMenu;
	}

	while(p != NULL)
	{
		if(hWnd == p->m_hWnd) return true;
		p = p->m_pSubPopupMenu;
	}

	return false;
}
