#include "xfc.h"

//////////////////////////////////////////////////////////////////////////////
// CStatusBar

#define STATUSBAR_HEIGHT		20

#define __PMF CStatusBar
BEGIN_MESSAGE_MAP(CStatusBar, CControlBar)
	ON_DRAW()
END_MESSAGE_MAP()
#undef __PMF

CStatusBar::CStatusBar()
{
	m_pPaneInfos = NULL;
	m_pRightEdges = NULL;
}

CStatusBar::~CStatusBar()
{
	if(m_pPaneInfos != NULL)
	{
		delete [] m_pPaneInfos;
		delete [] m_pRightEdges;
	}

	DestroyWindow();
}

BOOL CStatusBar::Create(CFrameWnd* pParentWnd, DWORD dwStyle, UINT nID)
{
	m_nPaneCount = 0;

	if(CWnd::Create("", dwStyle,
		0,0,100,32,	// default size
		pParentWnd, NULL))
	{
		pParentWnd->m_pStatusBar = this;
		return true;
	}
	
	return false;
}
int CStatusBar::GetHeight() const
{
	return STATUSBAR_HEIGHT;
}

void CStatusBar::OnDraw(CDC* pDC)
{
	int c1, c2;
	int x=1, x2;

	XClearWindow(XfcGetDisplay(), m_hWnd);

	for(int n=0; n<m_nPaneCount; n++)
	{
		x2 = m_pRightEdges[n] -2;
		if(x2 < 0)
		{
			x2 = m_nBarWidth -2;
		}

		switch(m_pPaneInfos[n].nStyle)
		{
		case SBPS_NOBORDERS:
			goto DrawTextNow;
		case SBPS_POPOUT:
			c1 = CColor::darkGray;
			c2 = CColor::white;
			break;
		default:
			c1 = CColor::white;
			c2 = CColor::darkGray;
		}

		pDC->SetFgColor(c1);
		pDC->DrawLine(x2, 1, x2, STATUSBAR_HEIGHT-1);
		pDC->DrawLine(x, STATUSBAR_HEIGHT -1,
			x2, STATUSBAR_HEIGHT -1);
		pDC->SetFgColor(c2);
		pDC->DrawLine(x, 1, x2-1, 1);
		pDC->DrawLine(x, 1, x, STATUSBAR_HEIGHT -2);

DrawTextNow:
		if(m_pPaneInfos[n].pszText != NULL)
		{
			pDC->SetFgColor(CColor::black);
			pDC->DrawString(x +2, STATUSBAR_HEIGHT-6,
				m_pPaneInfos[n].pszText);
		}

		x = x2 +3;
		if(x >= m_nBarWidth) break;
	}
}

/////////////////////////////////////////////////////////////////////////////

void CStatusBar::SetWidth(int cx)
{
	m_nBarWidth = cx;

	m_pRightEdges[m_nPaneCount -1] = -1;
	for(int n=(m_nPaneCount-2); n>=0; n--)
	{
		cx -= m_pPaneInfos[n+1].cxWidth;
		m_pRightEdges[n] = cx;
	}
}

BOOL CStatusBar::SetIndicators(const UINT *pIDArray, int nIDCount)
{
	ASSERT(nIDCount > 0);

	if(m_pPaneInfos) delete [] m_pPaneInfos;
	if(m_pRightEdges) delete [] m_pRightEdges;

	m_pPaneInfos = new CStbPaneInfo[nIDCount];
	m_pRightEdges = new int[nIDCount];

	for(int n=0; n<nIDCount; n++)
	{
		m_pPaneInfos[n].nID = pIDArray[n];
		m_pPaneInfos[n].nStyle = 0;
		m_pPaneInfos[n].cxWidth = 0;
		m_pPaneInfos[n].pszText = NULL;
		m_pRightEdges[n] = -1;
	}

	m_nPaneCount = nIDCount;

	return TRUE;
}

void CStatusBar::SetPaneInfo(int nIndex, UINT nID, UINT nStyle, LPCTSTR pszText)
{
	if(nIndex >= m_nPaneCount)
		return;

	m_pPaneInfos[nIndex].nID = nID;
	m_pPaneInfos[nIndex].nStyle = nStyle;

	if(pszText != NULL)
	{
		m_pPaneInfos[nIndex].pszText = pszText;
		m_pPaneInfos[nIndex].cxWidth = 6 + XTextWidth(
			CFont::s_pFontStruct_Normal, pszText, strlen(pszText));
	}
	else
	{
		m_pPaneInfos[nIndex].cxWidth = 0;
	}
}

void CStatusBar::SetPaneInfo(int nIndex, UINT nID, UINT nStyle, int cxWidth)
{
	if(nIndex >= m_nPaneCount)
		return;

	m_pPaneInfos[nIndex].nID = nID;
	m_pPaneInfos[nIndex].nStyle = nStyle;
	m_pPaneInfos[nIndex].cxWidth = cxWidth;
}

BOOL CStatusBar::SetPaneText(int nIndex, LPCTSTR pszNewText, BOOL bUpdate)
{
	if(nIndex >= m_nPaneCount)
		return FALSE;

	m_pPaneInfos[nIndex].pszText = pszNewText;
	return TRUE;
}

void CStatusBar::SetPaneStyle(int nIndex, UINT nStyle)
{
	if(nIndex >= m_nPaneCount)
		return;

	m_pPaneInfos[nIndex].nStyle = nStyle;
}
