#include "xfc.h"

//////////////////////////////////////////////////////////////////////////////
// CStatusBar

#define __PMF CToolBar
BEGIN_MESSAGE_MAP(CToolBar, CControlBar)
	ON_DRAW()
	ON_MOUSEMOVE()
	ON_LBUTTONDOWN()
	ON_LBUTTONUP()
	ON_LEAVEWINDOW()
END_MESSAGE_MAP()
#undef __PMF

CToolBar::CToolBar()
{
	m_hPixmap = NULL;
	m_hpmMask = NULL;
	m_pBtnItems = NULL;
}

CToolBar::~CToolBar()
{
	if(m_pBtnItems) delete [] m_pBtnItems;

	if(m_hPixmap != NULL)
	{
		XFreePixmap(XfcGetDisplay(), m_hPixmap);
		if(m_hpmMask != NULL) XFreePixmap(XfcGetDisplay(), m_hpmMask);
	}

	DestroyWindow();
}

BOOL CToolBar::Create(CFrameWnd* pParentWnd, DWORD dwStyle, UINT nID)
{
	m_nButtonCount = 0;
	m_pSelectedItem = NULL;
	m_bButtonDown = false;

	if(CWnd::Create("", dwStyle,
		0,0,100,32,	// default size
		pParentWnd, NULL))
	{
		pParentWnd->m_pToolBar = this;
		return true;
	}
	
	return false;
}
int CToolBar::GetHeight() const
{
	return 24;
}

void CToolBar::SetWidth(int cx)
{
	m_nBarWidth = cx -1;
}

BOOL CToolBar::LoadBitmap(char *xpmdata[])
{
	XpmAttributes attr;

	if(m_hPixmap != NULL)
	{
		XFreePixmap(XfcGetDisplay(), m_hPixmap);
		if(m_hpmMask != NULL) XFreePixmap(XfcGetDisplay(), m_hpmMask);
	}

	attr.closeness = 40000; // Allow for "similar" colors
	attr.valuemask = XpmSize | XpmReturnPixels | XpmCloseness;

	if(XpmCreatePixmapFromData(XfcGetDisplay(), XfcGetRootWindow(), 
		xpmdata, &m_hPixmap, &m_hpmMask, &attr) != XpmSuccess)
	{
		return FALSE;
	}

	XpmFreeAttributes(&attr);

	return TRUE;
}

BOOL CToolBar::SetButtons(const UINT *pIDArray, int nIDCount)
{
	if(nIDCount != m_nButtonCount)
	{
		if(m_pBtnItems) delete [] m_pBtnItems;
		m_pBtnItems = new struct BtnItem[nIDCount];
		m_nButtonCount = nIDCount;
	}

	for(int n=0; n<nIDCount; n++)
	{
//		m_pBtnItems[n].left = 0;
		m_pBtnItems[n].right = 0;
		m_pBtnItems[n].tbb.fsStyle = TBSTYLE_BUTTON;
	}

	return TRUE;
}

void CToolBar::SetButtonInfo(int nIndex, UINT nID, UINT nStyle, int iImage)
{
	int x, n;

	if(nIndex > m_nButtonCount)
		return;

	m_pBtnItems[nIndex].tbb.idCommand = nID;
	m_pBtnItems[nIndex].tbb.fsState = HIWORD(nStyle);
	m_pBtnItems[nIndex].tbb.fsStyle = LOWORD(nStyle);
	m_pBtnItems[nIndex].tbb.iBitmap = iImage;

	x = 0;
	for(n=0; n<m_nButtonCount; n++)
	{
		m_pBtnItems[n].left = x;
		switch(m_pBtnItems[n].tbb.fsStyle)
		{
		case TBSTYLE_BUTTON:
			x += 16 + 6;
			break;
		case TBSTYLE_SEP:
			x += 7;
			break;
		}

		m_pBtnItems[n].right = x++;
	}
}

void CToolBar::OnDraw(CDC* pDC)
{
	pDC->SetFgColor(CColor::darkGray);
	pDC->DrawLine(0,0, m_nBarWidth,0);
	pDC->SetFgColor(CColor::white);
	pDC->DrawLine(0,1, m_nBarWidth,1);

	for(int n=0; n<m_nButtonCount; n++)
	{
		int x = m_pBtnItems[n].left;
		if(m_pBtnItems[n].tbb.fsStyle == TBSTYLE_BUTTON)
		{
			XCopyArea(XfcGetDisplay(), m_hPixmap, m_hWnd,
				pDC->GetSafeHdc(),
				m_pBtnItems[n].tbb.iBitmap * 16, 0, 16, 15,
				x +3, 5);
		}
		else if(m_pBtnItems[n].tbb.fsStyle == TBSTYLE_SEP)
		{
			pDC->SetFgColor(CColor::darkGray);
			pDC->DrawLine(x+3, 4, x+3, 21);
			pDC->SetFgColor(CColor::white);
			pDC->DrawLine(x+4, 4, x+4, 21);
		}
	}
}

//////////////////////////////////////////////////////////////////////////////


struct CToolBar::BtnItem *CToolBar::GetBtnItemByPos(int x, int y)
{
	if(y <= 1 || y >= 22)
		return NULL;

	for(int n=0; n<m_nButtonCount; n++)
	{
		struct CToolBar::BtnItem *pItem = &m_pBtnItems[n];

		if(x < pItem->right)
		if(pItem->tbb.fsStyle == TBSTYLE_BUTTON)
			return pItem;
		else
			return NULL;
	}

	return NULL;
}

void CToolBar::DrawEdge(CDC* pDC, struct BtnItem* pItem, bool bDraw)
{
	UINT c1, c2;
ASSERT(pItem != NULL);

	if(bDraw == false)
	{
		// erase edge
		pDC->SetFgColor(CColor::lightGray);
		XDrawRectangle(XfcGetDisplay(), m_hWnd, pDC->m_hDC,
			pItem->left, 2, 22, 21);

		return;
	}
	else if(m_bButtonDown)
	{
		c1 = CColor::darkGray;
		c2 = CColor::white;
	}
	else
	{
		c1 = CColor::white;
		c2 = CColor::darkGray;
	}

	// draw edge
	pDC->SetFgColor(c1);
	pDC->DrawLine(pItem->left, 2, pItem->right, 2); 
	pDC->DrawLine(pItem->left, 2, pItem->left, 23); 
	pDC->SetFgColor(c2);
	pDC->DrawLine(pItem->left, 23, pItem->right, 23); 
	pDC->DrawLine(pItem->right, 2, pItem->right, 23); 
}

void CToolBar::OnMouseMove(UINT nFlags, int x, int y)
{
	BtnItem* pItem = GetBtnItemByPos(x, y);

	if(m_bButtonDown)
	{
		if((pItem != m_pSelectedItem && m_bSunkenDrawn == false)
		|| (pItem == m_pSelectedItem && m_bSunkenDrawn == true))
			return;

		CClientDC dc(this);

		XClearArea(XfcGetDisplay(), m_hWnd,
			m_pSelectedItem->left +3, 4, 18, 17, False);

		m_bSunkenDrawn = !m_bSunkenDrawn;
		DrawEdge(&dc, m_pSelectedItem, m_bSunkenDrawn);

		if(m_bSunkenDrawn == true)
		{
			XCopyArea(XfcGetDisplay(), m_hPixmap, m_hWnd,
				dc.GetSafeHdc(),
				m_pSelectedItem->tbb.iBitmap * 16, 0, 16, 15,
				m_pSelectedItem->left +4, 6);
		}
		else
		{
			XCopyArea(XfcGetDisplay(), m_hPixmap, m_hWnd,
				dc.GetSafeHdc(),
				m_pSelectedItem->tbb.iBitmap * 16, 0, 16, 15,
				m_pSelectedItem->left +3, 5);
		}
	}
	else
	{
		if(pItem == m_pSelectedItem) return;

		CClientDC dc(this);
		if(m_pSelectedItem != NULL)
			DrawEdge(&dc, m_pSelectedItem, false);
		
		if(pItem != NULL)
		{
			m_pSelectedItem = pItem;
			DrawEdge(&dc, pItem, true);
		}
	}
}

void CToolBar::OnLButtonDown(UINT nFlags, int x, int y)
{
	BtnItem* pItem = GetBtnItemByPos(x, y);
	if(pItem == NULL) return;

	if(pItem != m_pSelectedItem && m_pSelectedItem != NULL)
	{
		// erase previous
		CClientDC dc(this);
		DrawEdge(&dc, m_pSelectedItem, false);
	}

	m_pSelectedItem = pItem;
	m_bButtonDown = true;
	m_bSunkenDrawn = false;

	OnMouseMove(nFlags, x, y);
}

void CToolBar::OnLButtonUp(UINT nFlags, int x, int y)
{
	if(m_bButtonDown == false)
		return;

	BtnItem* pItem = GetBtnItemByPos(x, y);

	if(pItem == m_pSelectedItem)
	{
		XFCMSG msg;

		msg.window = m_pParentWnd->GetSafeHwnd();
		msg.message = XWM_COMMAND;
		msg.xevent.pad[0] = pItem->tbb.idCommand;
		msg.xevent.pad[1] = 0;

		XfcPostMessage(&msg);
	}

	if(m_bSunkenDrawn)
	{
		CClientDC dc(this);
		DrawEdge(&dc, m_pSelectedItem, false);
	}

	m_pSelectedItem = NULL;
	m_bButtonDown = false;
}

void CToolBar::OnLeaveWindow()
{
	if(m_pSelectedItem != NULL)
	{
		CClientDC dc(this);
		DrawEdge(&dc, m_pSelectedItem, false);
		m_bSunkenDrawn = false;

		if(m_bButtonDown == false)
			m_pSelectedItem = NULL;
	}
}
