#include "xfc.h"
#include <signal.h>

// Holder of Application data pointers
XfcModuleState xfcModuleState;

/*****************************************************************************/
// some of Global Functions

void XfcPostQuitMessage(int nExitCode)
{
	XFCMSG xmsg;

	xmsg.message = XWM_QUIT;
	xmsg.xevent.pad[0] = nExitCode;

	XfcGetApp()->PostMessage(&xmsg);
}

void XfcPostMessage(XFCMSG* pMsg)
{
	XfcGetApp()->PostMessage(pMsg);
}

/*****************************************************************************/

CWinApp::CWinApp(LPCTSTR pszAppName)
{
	if(pszAppName != NULL)
	{
		XfcGetAppName() = pszAppName;
	}

	m_pMenuTracker = NULL;

	// message queue reset
	m_nMsgQueueFront = m_nMsgQueueRear = 0;
	m_pMsgCur = &m_msgQueue[0];

	// self register
	xfcModuleState.m_pCurrentWinApp = this;
	xfcModuleState.m_pmapWnd = new CXidMap_CWnd();
}

CWinApp::~CWinApp()
{
	delete xfcModuleState.m_pmapWnd;
}

bool CWinApp::InitApplication()
{
	return true;
}

bool CWinApp::InitInstance()
{
	return true;
}

int CWinApp::ExitInstance()
{
	return m_pMsgCur->xevent.pad[0];
}

///////////////////////////////////////////////////////////////////////////////

void CWinApp::SetMenuTracker(CMenuTracker* pMT)
{
	if(m_pMenuTracker != NULL)
	{
		delete m_pMenuTracker;
	}

	m_pMenuTracker = pMT;
}

void CWinApp::AddModalLeader(CDialog* pModal)
{
	ASSERT(m_pModalLeader == NULL);	// 1234
	m_pModalLeader = pModal;
}

void CWinApp::RemoveModalLeader()
{
	ASSERT(m_pModalLeader != NULL);
	m_pModalLeader = NULL;
}

///////////////////////////////////////////////////////////////////////////////

void CWinApp::PostMessage(XFCMSG* pMsg)
{
	// Check if queue is full, then deliver first
	if((m_nMsgQueueRear +1) % XFC_MSGQUEUE_MAXSIZE == m_nMsgQueueFront)
	{
		m_pMsgCur = &m_msgQueue[m_nMsgQueueFront];
		m_nMsgQueueFront = ++m_nMsgQueueFront % XFC_MSGQUEUE_MAXSIZE;
		DispatchMessage(m_pMsgCur);
	}

	// push into queue
	memcpy(&m_msgQueue[m_nMsgQueueRear], pMsg, sizeof(XFCMSG));
	m_nMsgQueueRear = ++m_nMsgQueueRear % XFC_MSGQUEUE_MAXSIZE;
}
/*
void CWinApp::OnQueueFull()
{
	// Check if queue is full, then deliver first
	if((m_nMsgQueueRear +1) % XFC_MSGQUEUE_MAXSIZE == m_nMsgQueueFront)
	{
		m_pMsgCur = &m_msgQueue[m_nMsgQueueFront];
		m_nMsgQueueFront = ++m_nMsgQueueFront % XFC_MSGQUEUE_MAXSIZE;
		DispatchMessage(m_pMsgCur);
	}
}
*/
// If a message is available, the return value is nonzero (true)
// If no messages are available, the return value is zero (false)
bool CWinApp::PeekMessage(XFCMSG* pMsg)
{
	XFCMSG msg;

	if(m_nMsgQueueFront == m_nMsgQueueRear)	// queue is empty
	{
		if(XPending(XfcGetDisplay()) <= 0)
			return false;	// and there's no pending X event

		XNextEvent(XfcGetDisplay(), &msg.xevent);
		if(TranslateMessage(&msg) == false) // not translated
			return false;
	}

	memcpy(pMsg, m_pMsgCur = &m_msgQueue[m_nMsgQueueFront], sizeof(XFCMSG));
	return true;
}

// If the function retrieves a message other than WM_QUIT, the return value is nonzero.
// If the function retrieves the XWM_QUIT message, the return value is zero
bool CWinApp::GetMessage(XFCMSG* pMsg)
{
	XFCMSG msg;

	if(m_nMsgQueueFront == m_nMsgQueueRear)	// queue is empty
	{
		do XNextEvent(XfcGetDisplay(), &msg.xevent);
		while(TranslateMessage(&msg) == false);
	}

	memcpy(pMsg, m_pMsgCur = &m_msgQueue[m_nMsgQueueFront], sizeof(XFCMSG));
	m_nMsgQueueFront = ++m_nMsgQueueFront % XFC_MSGQUEUE_MAXSIZE;

	return (pMsg->message != XWM_QUIT);
}

bool CWinApp::TranslateMessage(XFCMSG* pMsg)
{
	// match the xevent type to our message id
	pMsg->message = 0;
	switch(pMsg->xevent.type)
	{
	case KeyPress:
		pMsg->message = XWM_KEYDOWN;
		break;
	case KeyRelease:
		pMsg->message = XWM_KEYUP;
		break;

	case ButtonPress:
		if(pMsg->xevent.xbutton.button == Button1)
			pMsg->message = XWM_LBUTTONDOWN;
		else if(pMsg->xevent.xbutton.button == Button3)
			pMsg->message = XWM_RBUTTONDOWN;
		break;

	case ButtonRelease:
		if(pMsg->xevent.xbutton.button == Button1)
			pMsg->message = XWM_LBUTTONUP;
		else if(pMsg->xevent.xbutton.button == Button3)
			pMsg->message = XWM_RBUTTONUP;
		break;

	case MotionNotify:
		pMsg->message = XWM_MOUSEMOVE;
		break;

	case EnterNotify:
		pMsg->message = XWM_ENTERWINDOW;
		break;
	case LeaveNotify:
		pMsg->message = XWM_LEAVEWINDOW;
		break;

	case FocusIn:
	case FocusOut:
	case KeymapNotify:
		break;

	case Expose:
		if(pMsg->xevent.xexpose.count > 0)
			break;	// more Expose exist
/*
		pWnd->m_xfc_rectInvalidated.left =
			pMsg->xevent.xexpose.x - pWnd->m_xfc_ptClientOffset.x;
		pWnd->m_xfc_rectInvalidated.top =
			pMsg->xevent.xexpose.y - pWnd->m_xfc_ptClientOffset.y;
		pWnd->m_xfc_rectInvalidated.right =
			pWnd->m_xfc_rectInvalidated.left + pMsg->xevent.xexpose.width;
		pWnd->m_xfc_rectInvalidated.bottom =
			pWnd->m_xfc_rectInvalidated.top + pMsg->xevent.xexpose.height;
*/
		pMsg->message = XWM_PAINT;
		break;

	case GraphicsExpose:
	case VisibilityNotify:
	case NoExpose:
		break;

	case CreateNotify:
		break;

	case DestroyNotify:
		pMsg->message = XWM_DESTROY;
		break;

	case UnmapNotify:
	case MapNotify:
	case MapRequest:
	case ReparentNotify:
		break;

	case ConfigureNotify:
		pMsg->message = XWM_SIZE;
		break;

	case ConfigureRequest:
	case GravityNotify:
	case ResizeRequest:
	case CirculateNotify:
	case CirculateRequest:
	case PropertyNotify:
	case SelectionClear:
	case SelectionRequest:
	case SelectionNotify:
	case ColormapNotify:
		break;

	case ClientMessage:
		if(pMsg->xevent.xclient.format == 32
		&& pMsg->xevent.xclient.data.l[0] == (signed) CWnd::s_wmDeleteWindow)
		{
			pMsg->message = XWM_CLOSE;
		}
		break;

	case MappingNotify:
		break;

	default:
		// not reachable (maybe)
		TRACE("No XEvent case for %d\n", pMsg->xevent.type);
		break;
	}

	if(pMsg->message == 0)
	{
//static char *event_names[] = { "", "", "KeyPress", "KeyRelease", "ButtonPress", "ButtonRelease", "MotionNotify", "EnterNotify", "LeaveNotify", "FocusIn", "FocusOut", "KeymapNotify", "Expose", "GraphicsExpose", "NoExpose", "VisibilityNotify", "CreateNotify", "DestroyNotify", "UnmapNotify", "MapNotify", "MapRequest", "ReparentNotify", "ConfigureNotify", "ConfigureRequest", "GravityNotify", "ResizeRequest", "CirculateNotify", "CirculateRequest", "PropertyNotify", "SelectionClear", "SelectionRequest", "SelectionNotify", "ColormapNotify", "ClientMessage", "MappingNotify" }; TRACE("Not-filtered event(%x - %s(%d))\n", pMsg->xevent.xany.window, event_names[pMsg->xevent.type], pMsg->xevent.type);
		return false;
	}
	
	if(pMsg->message != XWM_PAINT && m_pMenuTracker != NULL)
	{
		if(!m_pMenuTracker->IsOurMessage(pMsg->xevent.xany.window))
			return false;
/*
		{
if(pMsg->xevent.type == MotionNotify) {
TRACE("motion wnd:%x, subw:%x (%d,%d,%d,%d)\n", pMsg->xevent.xmotion.root, pMsg->xevent.xmotion.subwindow, pMsg->xevent.xmotion.x, pMsg->xevent.xmotion.y, pMsg->xevent.xmotion.x_root, pMsg->xevent.xmotion.y_root); 
}
else TRACE("cancel msg: %X\n", pMsg->xevent.xany.window);
			return false;
		}
*/
	}

	// push into queue
	pMsg->window = pMsg->xevent.xany.window;
	memcpy(&m_msgQueue[m_nMsgQueueFront], pMsg, sizeof(XFCMSG));
	m_nMsgQueueRear = ++m_nMsgQueueRear % XFC_MSGQUEUE_MAXSIZE;

	return true;
}

void CWinApp::DispatchMessage(XFCMSG* pMsg)
{
	CWnd* pWnd = XfcGetModuleState()->m_pmapWnd->Find(pMsg->window);

	if(pWnd == NULL)
	{
		TRACE("DispatchMessage(event:%d): CWnd(hwnd:%X) not found.\n", pMsg->xevent.type, pMsg->xevent.xany.window);
		return;
	}

	if(m_pModalLeader != NULL && pMsg->message != XWM_PAINT)
	{
		if(m_pModalLeader->IsMyChild(pWnd) == false)
			return;	// ignore
	}

	pWnd->WindowProc(pMsg->message, &pMsg->xevent);
}

bool CWinApp::PumpMessage()
{
	if(!GetMessage(m_pMsgCur))
	{
		TRACE("CWinApp::PumpMessage - XWM_QUIT : gracefully quit.\n");
		return false;
	}

	DispatchMessage(m_pMsgCur);
	return true;
}

int CWinApp::Run()
{
	XFCMSG msg;

	while(GetMessage(&msg)) DispatchMessage(&msg);
	return ExitInstance();
/*
	while(true)
	{
		while(!PeekMessage());

		do
		{
			if(!PumpMessage())
				return ExitInstance();
		} while(PeekMessage());
	}
*/
	ASSERT(false);	// not reachable
}

/////////////////////////////////////////////////////////////////////////////

void signalFunc(int nSigNum)
{
	printf("Got Signal=%d\n", nSigNum);
	XfcPostQuitMessage(1);
}

int main(int argc, char *argv[])
{
	int nReturnCode = -1;
	CWinApp* pApp = XfcGetApp();

	signal(SIGTERM, signalFunc);
	signal(SIGINT, signalFunc);

#ifdef _DEBUG
	_Xdebug = 1;
#endif
	xfcModuleState.pDisplay = XOpenDisplay(NULL);
	if(XfcGetDisplay() == NULL)
	{
		printf("Can't connect to X server %s\n", XDisplayName(NULL));
		return -1;
	}

	// Set Default App Name (executable)
/*
	{
		char *p;
		if((p = strrchr(argv[0], '/')) != NULL)
			XfcGetAppName() = p;
		else
			XfcGetAppName() = argv[0];
	}
*/
	XfcGetAppName() = "XFC App";

	xfcModuleState.nDefaultScreen = XDefaultScreen(XfcGetDisplay());
	xfcModuleState.hRootWindow = XRootWindow(XfcGetDisplay(),
		XfcGetDefaultScreen());
	xfcModuleState.pDefaultVisual = XDefaultVisual(XfcGetDisplay(),
		XfcGetDefaultScreen());
	xfcModuleState.nDefaultDepth = XDefaultDepth(XfcGetDisplay(),
		XfcGetDefaultScreen());

	if(CColor::_Initialize() == false
	|| CFont::_Initialize() == false) return -1;

// CWnd static member initialization
	CWnd::s_wmDeleteWindow = XInternAtom(XfcGetDisplay(),
		"WM_DELETE_WINDOW", False);
	if(CWnd::s_wmDeleteWindow == None)
	{
		puts("WM_DELETE_WINDOW atom failed.");
		return -1;
	}

	if(!pApp->InitApplication())
		goto InitFailure;

	if(!pApp->InitInstance())
	{
		if(pApp->m_pMainWnd != NULL)
		{
			TRACE0("Warning: Destroying non-NULL m_pMainWnd\n");
			pApp->m_pMainWnd->DestroyWindow();
		}

		nReturnCode = pApp->ExitInstance();
		goto InitFailure;
	}

	nReturnCode = pApp->Run();

InitFailure:
	CFont::_Release();
	CColor::_Release();
	XCloseDisplay(XfcGetDisplay());

	return nReturnCode;
}
