#include "xfc.h"

Atom CWnd::s_wmDeleteWindow = NULL;

CWnd::CWnd()
{
	m_hWnd = NULL;
}

CWnd::~CWnd()
{
}

bool CWnd::PreCreateWindow(CREATESTRUCT& cs)
{
	return true;
}

bool CWnd::PreCreateWindow_VisualInfo(XVisualInfo& vi)
{
	return true;
}

bool CWnd::Create(LPCTSTR pszWindowName, DWORD dwStyle,
	int x, int y, int cx, int cy,
	CWnd* pWndParent, void* pParams)
{
	XVisualInfo vi;
	unsigned int nBorderWidth =0;
	bool bSetDwEvent = false;
	bool bSendWMCREATE = false;
	CREATESTRUCT cs;


	ASSERT(m_hWnd == NULL);

	vi.screen = XfcGetDefaultScreen();
	vi.visual = XfcGetDefaultVisual();
	vi.depth = XfcGetDefaultDepth();

	if(pWndParent == NULL)
		cs.hwndParent = XfcGetRootWindow();
	else
		cs.hwndParent = pWndParent->GetSafeHwnd();

	cs.pCreateParams = pParams;
	cs.x = x;
	cs.y = y;
	cs.cx = cx;
	cs.cy = cy;
	cs.nSwaMask = CWBackPixel | CWEventMask | CWDontPropagate;
	cs.swa.background_pixel = CColor::lightGray;
	cs.swa.event_mask = NoEventMask;

	if(dwStyle & WS_BORDER)
	{
		nBorderWidth = 1;
		cs.nSwaMask |= CWBorderPixel;
		cs.swa.border_pixel = XBlackPixel(XfcGetDisplay(), vi.screen);
	}
	if(dwStyle & WS_POPUP)
	{
		cs.nSwaMask |= CWOverrideRedirect;
		cs.swa.override_redirect = True;
	}
	if(dwStyle & WS_SYSMENU)
	{
//		cs.swa.event_mask |= OwnerGrabButtonMask;
		cs.nSwaMask |= CWSaveUnder;
		cs.swa.save_under = True;
	}

	const AFX_MSGMAP* pMessageMap = GetMessageMap();
	while(pMessageMap != NULL)
	{
		for(int a=0; pMessageMap->pEntries[a].nSig != XfcSig_end; a++)
		{
			switch(pMessageMap->pEntries[a].nMessage)
			{
			case XWM_CREATE:
				bSendWMCREATE = true;
				break;

			case XWM_CLOSE:
				bSetDwEvent = true;
				break;

			case XWM_LBUTTONDOWN:
			case XWM_RBUTTONDOWN:
				cs.swa.event_mask |= ButtonPressMask;
				break;

			case XWM_LBUTTONUP:
			case XWM_RBUTTONUP:
				cs.swa.event_mask |= ButtonReleaseMask;
				break;

			case XWM_MOUSEMOVE:
				cs.swa.event_mask |= ButtonMotionMask;
				cs.swa.event_mask |= PointerMotionMask;
				break;

			case XWM_KEYDOWN:
			case XWM_CHAR:
				cs.swa.event_mask |= KeyPressMask;
				break;

			case XWM_KEYUP:
				cs.swa.event_mask |= KeyReleaseMask;
				break;

			case XWM_PAINT:
				cs.swa.event_mask |= ExposureMask;
				break;

			case XWM_ENTERSIZEMOVE:
				cs.swa.event_mask |= StructureNotifyMask;
				break;

			case XWM_ENTERWINDOW:
				cs.swa.event_mask |= EnterWindowMask;
				break;
			case XWM_LEAVEWINDOW:
				cs.swa.event_mask |= LeaveWindowMask;
				break;
				
			case XWM_SIZE:
				cs.swa.event_mask |= StructureNotifyMask;
				break;

			case XWM_SETFOCUS:
			case XWM_KILLFOCUS:
				cs.swa.event_mask |= FocusChangeMask;
				break;

			// no proper event handler
			case XWM_DESTROY:
				break;

			default:
				TRACE("Warning: un-mapped WM_? (%04X)\n",
					pMessageMap->pEntries[a].nMessage);
			}
		}
		pMessageMap = pMessageMap->pBaseMap;
	}
	cs.swa.do_not_propagate_mask = KeyPressMask | KeyReleaseMask
		| ButtonMotionMask | ButtonPressMask | PointerMotionMask
		| Button1MotionMask | Button2MotionMask | Button3MotionMask
		| Button4MotionMask | Button5MotionMask;

	PreCreateWindow(cs);
	PreCreateWindow_VisualInfo(vi);

	HWND hWnd = XCreateWindow(XfcGetDisplay(), cs.hwndParent,
		cs.x, cs.y, cs.cx, cs.cy, nBorderWidth, vi.screen,
		InputOutput, vi.visual, cs.nSwaMask, &cs.swa);

	if(hWnd == NULL) return false;

	m_pParentWnd = pWndParent;

	if(pszWindowName)
		XStoreName(XfcGetDisplay(), hWnd, pszWindowName);

	if(bSetDwEvent)
	{
		// we'll get window close notice from WindowManager
		XSetWMProtocols(XfcGetDisplay(), hWnd, &s_wmDeleteWindow, 1);
	}

	// Attach HWND
	CXidMap_CWnd* pMap = XfcGetModuleState()->m_pmapWnd;
	ASSERT(pMap != NULL);
	pMap->Add(m_hWnd = hWnd, this);

	if(dwStyle & WS_VISIBLE)
	{
		ShowWindow(SW_SHOW);
	}

	// Send XWM_CREATE message
	if(bSendWMCREATE)
	{
		XEvent event;

		memcpy(&event, &cs, sizeof(cs));
		if(WindowProc(XWM_CREATE, &event) != 0)
		{
			TRACE("CWnd::OnCreate() failed.");
			pMap->Remove(m_hWnd);
			return false;
		}
	}

	// post initial XWM_SIZE message
	if(cs.swa.event_mask & StructureNotifyMask)
	{
		XFCMSG msg;
		XConfigureEvent* pEvent = (XConfigureEvent*) &msg.xevent;

		msg.window = m_hWnd;
		msg.message = XWM_SIZE;
		pEvent->width = cs.cx;
		pEvent->height = cs.cy;

		XfcGetApp()->PostMessage(&msg);
	}

	return true;
}

//////////////////////////////////////////////////////////////////////////////

LRESULT CWnd::SendMessage(UINT message, XEvent* pEvent)
{
	LRESULT lResult;

	OnXfcMsg(message, pEvent, &lResult);

	return lResult;
}
/*
BOOL CWnd::PostMessage(XFCMSG* pMsg)
{
	ASSERT(m_hWnd != NULL);
	return ::PostMessage(m_hWnd, message, wParam, lParam);
}
*/

//////////////////////////////////////////////////////////////////////////////

LRESULT CWnd::WindowProc(UINT message, XEvent* pEvent)
{
	// OnWndMsg does most of the work, except for DefWindowProc call
	LRESULT lResult = 0;
	if(!OnXfcMsg(message, pEvent, &lResult))
		lResult = DefWindowProc(message, pEvent);
	return lResult;
}

LRESULT CWnd::DefWindowProc(UINT message, XEvent* pEvent)
{
	TRACE("CWnd::DefaultWndProc(%d)\n", message);

	return 0;
}

bool CWnd::OnXfcMsg(UINT message, XEvent* pEvent, LRESULT* pResult)
{
	const AFX_MSGMAP* pMessageMap;
	const AFX_MSGMAP_ENTRY* pEntry;
	union MessageMapFunctions mmf;
	LRESULT lResult;

	if(message == XWM_COMMAND)
	{
		if(OnCommand(pEvent->pad[0], pEvent->pad[1]))
		{
			lResult = 1;
			goto LReturnTrue;
		}
		return false;
	}

	pMessageMap = GetMessageMap();
	for(;pMessageMap != NULL; pMessageMap = pMessageMap->pBaseMap)
	{
		// check BEGIN_MESSAGE_MAP(CMyWnd, CMyWnd);
		ASSERT(pMessageMap != pMessageMap->pBaseMap);
		// Find from MessageMap
		for(pEntry = pMessageMap->pEntries;
			pEntry->nSig != XfcSig_end; pEntry++)
		{
			if(pEntry->nMessage == message)
//			&& pEntry->nCode == nCode
//			&& nID >= pEntry->nID
//			&& nID <= pEntry->nLastID)
			{
				goto LDispatch;
			}
		}
	}

	return false;

LDispatch:
	mmf.pfn = pEntry->pfn;

	lResult = 0;
	switch(pEntry->nSig)
	{
	case XfcSig_IntPcreatestruct:
		lResult = (this->*mmf.pfn_IntPcreatestruct)(
			(CREATESTRUCT*)pEvent);
		break;
	case XfcSig_VoidUintIntInt_button:
		// LBUTTONDOWN, RBUTTONDOWN
		(this->*mmf.pfn_VoidUintIntInt_button)(pEvent->xbutton.state,
			pEvent->xbutton.x, pEvent->xbutton.y);
		break;
	case XfcSig_VoidUintIntInt_motion:
		// MOUSEMOVE
		(this->*mmf.pfn_VoidUintIntInt_motion)(pEvent->xmotion.state,
			pEvent->xmotion.x, pEvent->xmotion.y);
		break;
	case XfcSig_VoidIntInt_size:
		// SIZE
		(this->*mmf.pfn_VoidIntInt_size)(
			pEvent->xconfigure.width,
			pEvent->xconfigure.height);
		break;
	case XfcSig_VoidUintUintUint:
		// KEYDOWN
		{
			KeySym ks = XLookupKeysym(&pEvent->xkey, 0);
if(ks == 0) TRACE("no trans=%d\n", pEvent->xkey.keycode);
			(this->*mmf.pfn_VoidUintUintUint)(
				ks, 1, pEvent->xkey.state);
		}
		break;
	case XfcSig_VoidPcdc:
		// PAINT
		{
			static CDC dc;
			dc.m_hGC = XDefaultGC(XfcGetDisplay(), XfcGetDefaultScreen());
			dc.m_hDrawable = m_hWnd;
			(this->*mmf.pfn_VoidPcdc)(&dc);
		}
		break;
	case XfcSig_VoidVoid:
		(this->*mmf.pfn_VoidVoid)();
		break;
	}

LReturnTrue:
	if(pResult != NULL)
		*pResult = lResult;
	return true;
}

//////////////////////////////////////////////////////////////////////////////
// Virtual functions

bool CWnd::OnCommand(WPARAM wParam, LPARAM lParam)
{
	TRACE("CWnd(%x)::OnCommand(%d,%d)\n", m_hWnd, wParam, lParam);
	return true;
}

/*
BOOL CWnd::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	TRACE("CWnd::OnNotify(%d,%d,%p)\n", wParam, lParam, pResult);
	return true;
}
*/

//////////////////////////////////////////////////////////////////////////////

void CWnd::OnDestroy()
{
	TRACE("CWnd::OnDestory()\n");
}

//////////////////////////////////////////////////////////////////////////////

void CWnd::Invalidate(bool bErase)
{
	XFCMSG msg;

	msg.window = m_hWnd;
	msg.message = XWM_PAINT;

	XfcPostMessage(&msg);
}

void CWnd::UpdateWindow()
{
}

void CWnd::SetWindowText(LPCTSTR pszString)
{
	XStoreName(XfcGetDisplay(), m_hWnd, pszString);
}

int CWnd::GetWindowText(LPTSTR pszStringBuf, int nMaxCount) const
{
	Status ret;
	
	ret = XFetchName(XfcGetDisplay(), m_hWnd, &pszStringBuf);

	if(ret == 0) return 0;
	
	return strlen(pszStringBuf);
}

bool CWnd::ShowWindow(int nCmdShow)
{
	ASSERT(m_hWnd != NULL);

	Display* display = XfcGetDisplay();

	if(nCmdShow == SW_SHOW)
	{
		XMapWindow(display, m_hWnd);
	}
	else if(nCmdShow == SW_HIDE)
	{
		XUnmapWindow(display, m_hWnd);
	}
	else return false;

	return true;
}

void CWnd::MoveWindow(int x, int y, int cx, int cy, bool bRepaint)
{
	XMoveResizeWindow(XfcGetDisplay(), m_hWnd, x, y, cx, cy);
/*
	XWindowChanges xc;

	xc.x = x;
	xc.y = y;
	xc.width = cx;
	xc.height = cy;
	XConfigureWindow(XfcGetDisplay(), m_hWnd,
		CWX | CWY | CWWidth | CWHeight, &xc);
*/
}

void CWnd::MoveWindow(RECT* pRect, bool bRepaint)
{
	XMoveResizeWindow(XfcGetDisplay(), m_hWnd,
		pRect->left, pRect->top,
		pRect->right - pRect->left,
		pRect->bottom - pRect->top);
}

bool CWnd::DestroyWindow()
{
	int nResult;

	if(m_hWnd == NULL) return false;

	// Hide from the screen
	XUnmapWindow(XfcGetDisplay(), m_hWnd);
	// Send WM_DESTROY
	WindowProc(XWM_DESTROY, NULL);
	// Destroy Now
	nResult = XDestroyWindow(XfcGetDisplay(), m_hWnd);
	// Detach HWND
	CXidMap_CWnd* pMap = XfcGetModuleState()->m_pmapWnd;
	ASSERT(pMap != NULL);
	pMap->Remove(m_hWnd);

	return (nResult != BadWindow);
}

void CWnd::SetCursor(HCURSOR hCursor)
{
	XDefineCursor(XfcGetDisplay(), m_hWnd, hCursor);
}

void CWnd::SetCursorPos(int x, int y)
{
	XWarpPointer(XfcGetDisplay(), None, XfcGetRootWindow(),
		0,0,0,0, x, y);
}

bool CWnd::SetCapture()
{
TRACE("SetCapture(%x)\n", m_hWnd);
/*
	XGrabButton(XfcGetDisplay(), AnyButton,
		AnyModifier, m_hWnd, True,
		ButtonPressMask | ButtonReleaseMask | EnterWindowMask,
		GrabModeAsync, GrabModeAsync, None, None);
*/
	XGrabPointer(XfcGetDisplay(), m_hWnd, True,
		ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
		GrabModeAsync, GrabModeAsync,
		None, None, CurrentTime);
	XGrabKeyboard(XfcGetDisplay(), m_hWnd, False,
		GrabModeAsync, GrabModeAsync,
		CurrentTime);
//	XAllowEvents(XfcGetDisplay(), AsyncBoth, CurrentTime);

	return true;
}

bool CWnd::ReleaseCapture()
{
	XUngrabPointer(XfcGetDisplay(), CurrentTime);
	XUngrabKeyboard(XfcGetDisplay(), CurrentTime);
//	XUngrabButton(XfcGetDisplay(), Button1|Button2, AnyModifier, m_hWnd);

//	XAllowEvents(XfcGetDisplay(), ReplayPointer, CurrentTime);
//	XAllowEvents(XfcGetDisplay(), ReplayKeyboard, CurrentTime);
	XFlush(XfcGetDisplay());

	return true;
}

CWnd* CWnd::GetCapture()
{
	ASSERT(false);
	return NULL;
}

void CWnd::SetFocus()
{
	XSetInputFocus(XfcGetDisplay(), m_hWnd, RevertToParent, CurrentTime);
}

void CWnd::ClientToScreen(POINT* pPoint) const
{
	Window childWnd;

	XTranslateCoordinates(XfcGetDisplay(),
		m_hWnd, XfcGetRootWindow(),
		pPoint->x, pPoint->y,
		&pPoint->x, &pPoint->y, &childWnd);
}

void CWnd::ScreenToClient(POINT* pPoint) const
{
	Window childWnd;

	XTranslateCoordinates(XfcGetDisplay(),
		XfcGetRootWindow(), m_hWnd,
		pPoint->x, pPoint->y,
		&pPoint->x, &pPoint->y, &childWnd);
}

//////////////////////////////////////////////////////////////////////////////
// Message table implementation

#define __PMF CWnd
BEGIN_MESSAGE_MAP(CWnd, CCmdTarget)
	ON_DESTROY()
END_MESSAGE_MAP()
#undef __PMF

//////////////////////////////////////////////////////////////////////////////

