#include "xfc.h"

#define __PMF CStatic
BEGIN_MESSAGE_MAP(CStatic, CWnd)
	ON_DRAW()
END_MESSAGE_MAP()
#undef __PMF

CStatic::CStatic()
{
}

bool CStatic::Create(LPCTSTR pszText, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
	m_pszText = pszText;
	m_dwStyle = dwStyle;

	return CWnd::Create(NULL, 0,
		rect.left, rect.top,
		rect.right - rect.left, rect.bottom - rect.top,
		pParentWnd, NULL);
}

void CStatic::OnDraw(CDC* pDC)
{
	pDC->SetFont(CFont::s_hFont_Normal);
	pDC->SetFgColor(CColor::black);
	pDC->DrawString(0,10, m_pszText);
}

//////////////////////////////////////////////////////////////////////////////
// Button State

#define BUTTONSTATE_PUSHED		1
#define BUTTONSTATE_ENTERED		2

#define __PMF CButton
BEGIN_MESSAGE_MAP(CButton, CWnd)
	ON_MOUSEMOVE()
	ON_LBUTTONDOWN()
	ON_LBUTTONUP()
	ON_DRAW()
END_MESSAGE_MAP()
#undef __PMF

bool CButton::Create(LPCTSTR pszCaption, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
	m_pszCaption = pszCaption;
	m_dwStyle = dwStyle;
	m_nID = nID;
	m_pParentWnd = pParentWnd;

	m_bButtonDown = false;
	m_bDrawSunken = false;
	m_cxButton = rect.right - rect.left;
	m_cyButton = rect.bottom - rect.top;

	m_nItemXpos = (m_cxButton - XTextWidth(CFont::s_pFontStruct_Normal, pszCaption, strlen(pszCaption))) / 2;
	m_nItemYpos = (m_cyButton + CFont::s_pFontStruct_Normal->ascent + CFont::s_pFontStruct_Normal->descent -4) /2;
		
	return CWnd::Create(NULL, 0,
		rect.left, rect.top, m_cxButton, m_cyButton,
		pParentWnd, NULL);
}

void CButton::OnDraw(CDC* pDC)
{
	XClearWindow(XfcGetDisplay(), m_hWnd);

	if(m_bDrawSunken)
	{
		pDC->Rectangle(0,0, m_cxButton, m_cyButton);
		pDC->SetFgColor(CColor::black);
		pDC->DrawString(m_nItemXpos +1, m_nItemYpos +1, m_pszCaption);
		pDC->SetFgColor(CColor::darkGray);
		pDC->Rectangle(1,1, m_cxButton-1, m_cyButton-1);
	}
	else
	{
		pDC->SetFgColor(CColor::white);
		pDC->DrawLine(0,0, m_cxButton, 0);
		pDC->DrawLine(0,0, 0, m_cyButton);

		pDC->SetFgColor(CColor::black);
		pDC->DrawString(m_nItemXpos, m_nItemYpos, m_pszCaption);
		pDC->DrawLine(0, m_cyButton, m_cxButton, m_cyButton);
		pDC->DrawLine(m_cxButton, 0, m_cxButton, m_cyButton);

		pDC->SetFgColor(CColor::darkGray);
		pDC->DrawLine(1, m_cyButton-1, m_cxButton-1, m_cyButton-1);
		pDC->DrawLine(m_cxButton-1,1, m_cxButton-1, m_cyButton-1);
	}
}

bool CButton::PtInButton(int x, int y)
{
	return (x > 0 && y > 0 && x<m_cxButton && y<m_cyButton) ?
		true : false;
}

void CButton::OnMouseMove(UINT nFlags, int x, int y)
{
	if(m_bButtonDown == false) return;
	
	if(PtInButton(x, y))
	{
		if(m_bDrawSunken == false)
		{
			m_bDrawSunken = true;
		}
		else
			return;
	}
	else
	{
		if(m_bDrawSunken == true)
		{
			m_bDrawSunken = false;
		}
		else
			return;
			
	}

	Invalidate();
}

void CButton::OnLButtonDown(UINT nFlags, int x, int y)
{
	m_bButtonDown = true;
	m_bDrawSunken = true;
	Invalidate();
}

void CButton::OnLButtonUp(UINT nFlags, int x, int y)
{
	XFCMSG msg;

	if(m_bButtonDown == false) return;

	if(PtInButton(x, y))
	{
		// send WM_COMMAND notify
		msg.window = m_pParentWnd->GetSafeHwnd();
		msg.message = XWM_COMMAND;
		msg.xevent.pad[0] = m_nID;
		msg.xevent.pad[1] = GetSafeHwnd();

		XfcPostMessage(&msg);
	}

	m_bButtonDown = false;
	m_bDrawSunken = false;
	Invalidate();
}
