#ifndef __WINTYPES_H__
#define __WINTYPES_H__

#define PASCAL

typedef unsigned long		DWORD;
typedef unsigned char		BYTE;
typedef unsigned short		WORD;
typedef float			FLOAT;
typedef long			LONG;
typedef short			SHORT;
typedef unsigned int		UINT;
typedef void			VOID;

typedef bool			BOOL;
#define FALSE			false
#define TRUE			true

typedef char*			LPTSTR;
typedef const char*		LPCTSTR;
typedef void*			LPVOID;
typedef char			TCHAR;

typedef UINT WPARAM;
typedef LONG LPARAM;
typedef LONG LRESULT;

typedef Window	HWND;
typedef GC	HDC;
typedef Font	HFONT;
typedef Cursor	HCURSOR;

typedef struct tagRECT
{
	LONG	left;
	LONG	top;
	LONG	right;
	LONG	bottom;
} RECT;

typedef struct tagPOINT
{
	int x;
	int y;
} POINT;

typedef struct tagSIZE
{
	int cx;
	int cy;
} SIZE;

typedef struct tagRGBQUAD
{
	BYTE rgbBlue;
	BYTE rgbGreen;
	BYTE rgbRed;
	BYTE regReserved;
} RGBQUAD;

typedef struct tagBITMAPFILEHEADER
{
	WORD	bfType;
	DWORD	bfSize;
	WORD	bfReserved1;
	WORD	bfReserved2;
	DWORD	bfOffBits;
} BITMAPFILEHEADER;

typedef struct tagBITMAPINFOHEADER
{
	DWORD	biSize;
	LONG	biWidth;
	LONG	biHeight;
	WORD	biPlanes;
	WORD	biBitCount;
	DWORD	biCompression;
	DWORD	biSizeImage;
	LONG	biXPelsPerMeter;
	LONG	biYPelsPerMeter;
	DWORD	biClrUsed;
	DWORD	biClrImportant;
} BITMAPINFOHEADER;

/* The size of CREATESTRUCT structure must be less than that of XEvent
 * Currently CREATESTRUCT=92 bytes, XEvent=96 bytes (in 32bit env)
 */
typedef struct _CREATESTRUCT
{
	VOID*		pCreateParams;
	HWND		hwndParent;
	int		cy;
	int		cx;
	int		y;
	int		x;
	LONG		style;
	unsigned long	nSwaMask;
	XSetWindowAttributes swa;	// 60 bytes
} CREATESTRUCT;

// state component XButtonEvent, XMotionEvent
#define MK_LBUTTON          0x0100
#define MK_MBUTTON          0x0200
#define MK_RBUTTON          0x0400
#define MK_SHIFT            0x0001
#define MK_CONTROL          0x0004

#define MAKELONG(a, b)	((LONG) (((WORD) (a)) | ((DWORD) ((WORD) (b))) << 16))
#define HIWORD(l)	((WORD) (((DWORD) (l) >> 16) & 0xFFFF)) 
#define LOWORD(l)	((WORD) (l)) 
#define HIBYTE(w)	((BYTE) (((WORD) (w) >> 8) & 0xFF))
#define LOBYTE(w)	((BYTE) (w))

////////////////////////////////////////////////////////////
// ToolBar Style

typedef struct _TBBUTTON
{ 
	int iBitmap;
	int idCommand;
	BYTE fsState;
	BYTE fsStyle;
	DWORD dwData;
	int iString;
} TBBUTTON;

#define TBSTATE_ENABLED			0
#define TBSTATE_DISABLED		1
#define TBSTATE_CHECKED			2

#define TBSTYLE_BUTTON			0
#define TBSTYLE_SEP			1

#define TBBS_SEPARATOR		MAKELONG(TBSTYLE_SEP, 0)
#define TBBS_BUTTON		MAKELONG(TBSTYLE_BUTTON, 0)
#define TBBS_CHECKED		MAKELONG(0, TBSTATE_CHECKED)

////////////////////////////////////////////////////////////

// MessageBox() Flags
#define MB_OK				0x00000000L
#define MB_OKCANCEL			0x00000001L
#define MB_ABORTRETRYIGNORE		0x00000002L
#define MB_YESNOCANCEL			0x00000003L
#define MB_YESNO			0x00000004L
#define MB_RETRYCANCEL			0x00000005L
#define MB_ICONHAND			0x00000010L
#define MB_ICONQUESTION			0x00000020L
#define MB_ICONEXCLAMATION		0x00000030L
#define MB_ICONASTERISK			0x00000040L

// Dialog Box Command IDs
#define IDOK			1
#define IDCANCEL		2
#define IDABORT			3
#define IDRETRY			4
#define IDIGNORE		5
#define IDYES			6
#define IDNO			7
#define IDCLOSE			8
#define IDHELP			9

////////////////////////////////////////////////////////////
// StatusBar pane style

#define SBPS_NOBORDERS		0x10000000
#define SBPS_POPOUT		0x20000000
#define SBPS_OWNERDRAW		0x40000000
#define SBPS_RTLREADING		0x80000000

////////////////////////////////////////////////////////////
// Keyboard Code Table

#define VK_ESCAPE		XK_Escape
#define VK_RETURN		XK_Return
#define VK_SPACE		XK_Space
#define VK_BACK			XK_BackSpace
#define VK_TAB			XK_Tab
#define VK_END			XK_End
#define VK_HOME			XK_Home
#define VK_INSERT		XK_Insert
#define VK_DELETE		XK_Delete
#define VK_LEFT			XK_Left
#define VK_UP			XK_Up
#define VK_RIGHT		XK_Right
#define VK_DOWN			XK_Down

#define VK_A			XK_A

////////////////////////////////////////////////////////////
// Menu Flags

#define MF_DEFAULT		0x00010000L
#define MF_SYSMENU		0x00020000L
#define MF_HELP			0x00040000L

#define MF_SEPARATOR		0x00000001L
#define MF_STRING		0x00000002L
#define MF_BITMAP		0x00000004L
#define MF_OWNERDRAW		0x00000008L

#define MF_POPUP		0x00000010L
#define MF_MENUBARBREAK		0x00000020L
#define MF_MENUBREAK		0x00000040L

#define MF_ENABLED		0x00000000L
#define MF_UNCHECKED		0x00000000L
#define MF_CHECKED		0x00000100L
#define MF_GRAYED		0x00000200L
#define MF_DISABLED		0x00000400L

#define MF_BYCOMMAND		0x00001000L
#define MF_BYPOSITION		0x00002000L

////////////////////////////////////////////////////////////
// the followings values are come from windows directly

/*
 * Scroll Bar Constants
 */
#define SB_HORZ             0
#define SB_VERT             1
#define SB_CTL              2
#define SB_BOTH             3

/*
 * Scroll Bar Commands
 */
#define SB_LINEUP           0
#define SB_LINELEFT         0
#define SB_LINEDOWN         1
#define SB_LINERIGHT        1
#define SB_PAGEUP           2
#define SB_PAGELEFT         2
#define SB_PAGEDOWN         3
#define SB_PAGERIGHT        3
#define SB_THUMBPOSITION    4
#define SB_THUMBTRACK       5
#define SB_TOP              6
#define SB_LEFT             6
#define SB_BOTTOM           7
#define SB_RIGHT            7
#define SB_ENDSCROLL        8

/*
 * ShowWindow() Commands
 */
#define SW_HIDE             0
#define SW_SHOWNORMAL       1
#define SW_NORMAL           1
#define SW_SHOWMINIMIZED    2
#define SW_SHOWMAXIMIZED    3
#define SW_MAXIMIZE         3
#define SW_SHOWNOACTIVATE   4
#define SW_SHOW             5
#define SW_MINIMIZE         6
#define SW_SHOWMINNOACTIVE  7
#define SW_SHOWNA           8
#define SW_RESTORE          9
#define SW_SHOWDEFAULT      10
#define SW_MAX              10

/*
 * WM_SIZE message wParam values
 */
#define SIZE_RESTORED       0
#define SIZE_MINIMIZED      1
#define SIZE_MAXIMIZED      2
#define SIZE_MAXSHOW        3
#define SIZE_MAXHIDE        4

/*
 * Window Styles
 */
#define WS_OVERLAPPED       0x00000000L
#define WS_POPUP            0x80000000L
#define WS_CHILD            0x40000000L
#define WS_MINIMIZE         0x20000000L
#define WS_VISIBLE          0x10000000L
#define WS_DISABLED         0x08000000L
#define WS_CLIPSIBLINGS     0x04000000L
#define WS_CLIPCHILDREN     0x02000000L
#define WS_MAXIMIZE         0x01000000L
#define WS_CAPTION          0x00C00000L     /* WS_BORDER | WS_DLGFRAME  */
#define WS_BORDER           0x00800000L
#define WS_DLGFRAME         0x00400000L
#define WS_VSCROLL          0x00200000L
#define WS_HSCROLL          0x00100000L
#define WS_SYSMENU          0x00080000L
#define WS_THICKFRAME       0x00040000L
#define WS_GROUP            0x00020000L
#define WS_TABSTOP          0x00010000L

#define WS_MINIMIZEBOX      0x00020000L
#define WS_MAXIMIZEBOX      0x00010000L

#define WS_TILED            WS_OVERLAPPED
#define WS_ICONIC           WS_MINIMIZE
#define WS_SIZEBOX          WS_THICKFRAME
#define WS_TILEDWINDOW      WS_OVERLAPPEDWINDOW

/*
 * Common Window Styles
 */
#define WS_OVERLAPPEDWINDOW (WS_OVERLAPPED     | \
                             WS_CAPTION        | \
                             WS_SYSMENU        | \
                             WS_THICKFRAME     | \
                             WS_MINIMIZEBOX    | \
                             WS_MAXIMIZEBOX)

#define WS_POPUPWINDOW      (WS_POPUP          | \
                             WS_BORDER         | \
                             WS_SYSMENU)

#define WS_CHILDWINDOW      (WS_CHILD)

/*
 * Extended Window Styles
 */
#define WS_EX_DLGMODALFRAME     0x00000001L
#define WS_EX_NOPARENTNOTIFY    0x00000004L
#define WS_EX_TOPMOST           0x00000008L
#define WS_EX_ACCEPTFILES       0x00000010L
#define WS_EX_TRANSPARENT       0x00000020L
#define WS_EX_MDICHILD          0x00000040L
#define WS_EX_TOOLWINDOW        0x00000080L
#define WS_EX_WINDOWEDGE        0x00000100L
#define WS_EX_CLIENTEDGE        0x00000200L
#define WS_EX_CONTEXTHELP       0x00000400L

#define WS_EX_RIGHT             0x00001000L
#define WS_EX_LEFT              0x00000000L
#define WS_EX_RTLREADING        0x00002000L
#define WS_EX_LTRREADING        0x00000000L
#define WS_EX_LEFTSCROLLBAR     0x00004000L
#define WS_EX_RIGHTSCROLLBAR    0x00000000L

#define WS_EX_CONTROLPARENT     0x00010000L
#define WS_EX_STATICEDGE        0x00020000L
#define WS_EX_APPWINDOW         0x00040000L


#define WS_EX_OVERLAPPEDWINDOW  (WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE)
#define WS_EX_PALETTEWINDOW     (WS_EX_WINDOWEDGE | WS_EX_TOOLWINDOW | WS_EX_TOPMOST)


#endif
