#ifndef __XFCWIN_H__
#define __XFCWIN_H__

#define XfcGetDisplay()		(xfcModuleState.pDisplay)

#define FRIEND_MAIN()		friend int main(int argc, char* argv[]);

#include "xfctempl.h"

// Global Variable Structure
typedef struct _XfcModuleState
{
	CWinApp* m_pCurrentWinApp;
	LPCTSTR m_pszAppName;

	CXidMap_CWnd* m_pmapWnd;
	// Display Info
	Display* pDisplay;
	int nDefaultScreen;
	HWND hRootWindow;
	Visual* pDefaultVisual;
	int nDefaultDepth;
} XfcModuleState;

extern XfcModuleState xfcModuleState;

#define XfcGetDefaultScreen()	(xfcModuleState.nDefaultScreen)
#define XfcGetRootWindow()	(xfcModuleState.hRootWindow)
#define XfcGetDefaultVisual()	(xfcModuleState.pDefaultVisual)
#define XfcGetDefaultDepth()	(xfcModuleState.nDefaultDepth)

#define XfcGetModuleState()	(&xfcModuleState)
#define XfcGetApp()		(xfcModuleState.m_pCurrentWinApp)
#define XfcGetAppName()		(xfcModuleState.m_pszAppName)

typedef struct tagXFCMSG
{
	Window window;
	UINT message;
	XEvent xevent;
} XFCMSG;

///////////////////////////////////////////////////////////////////////////////

#define RGB(r,g,b)		(*CColor::s_pfnCCF)(r,g,b)

class CColor
{
public:
	static UINT white, black,
		gray, lightGray, darkGray,
		red, green, blue,
		cyan, magenta, yellow,
		darkRed, darkGreen, darkBlue,
		darkCyan, darkMagenta, darkYellow;

	static UINT (*s_pfnCCF)(BYTE r, BYTE g, BYTE b);
private:
	static Colormap s_hColormap;

	static bool _Initialize();
	static void _Release();

	// Color Combine Functions
	static UINT CCF_INDEXED(BYTE r, BYTE g, BYTE b);
	static UINT CCF_RGB888(BYTE r, BYTE g, BYTE b);
	static UINT CCF_RGB565(BYTE r, BYTE g, BYTE b);

	FRIEND_MAIN()
};

class CFont
{
public:
	static HFONT s_hFont_Normal;
	static HFONT s_hFont_Bold;
	static HFONT s_hFont_Small;
	static XFontStruct *s_pFontStruct_Normal;
	static XFontStruct *s_pFontStruct_Bold;
	static XFontStruct *s_pFontStruct_Small;

private:
	static bool _Initialize();
	static void _Release();

	FRIEND_MAIN()
};

class CDC
{
public:
	GC m_hGC;
#define m_hDC 	m_hGC
	Drawable m_hDrawable;

	inline HDC GetSafeHdc() const;

// operations
	void SetFgColor(UINT nColor);
	void SetBkColor(UINT nColor);
	void SetFont(HFONT hFont);

	void DrawPoint(int x, int y);
	void DrawLine(int x1, int y1, int x2, int y2);
	void Rectangle(int x1, int y1, int x2, int y2);
	void FillRect(short x, short y, WORD width, WORD height);
	void DrawString(int x, int y, LPCTSTR pszString);
};

class CPaintDC : public CDC
{
public:
	CPaintDC(CWnd* pWnd);
	~CPaintDC();

private:
	void _CreateDefaultGC();
	void _ReleaseDefaultGC();

	static GC s_hDefaultGC;

	FRIEND_MAIN()
};

class CClientDC : public CDC
{
public:
	CClientDC(CWnd* pWnd);
	~CClientDC();

private:
};

///////////////////////////////////////////////////////////////////////////////

class CCmdTarget
{
//	DECLARE_DYNAMIC(CCmdTarget)
protected:

public:
// Constructors
	CCmdTarget();

//	XfcModuleState* m_pModuleState;

protected:
//	friend class CView;

//	CView* GetRoutingView();
//	CFrameWnd* GetRoutingFrame();
	DECLARE_MESSAGE_MAP()		// base class - no {{ }} macros
};

class CWnd : public CCmdTarget
{
public:
	HWND m_hWnd;

// Constructor & other creator
	CWnd();
	virtual ~CWnd();

	virtual BOOL Create(LPCTSTR lpszWindowName, DWORD dwStyle,
		int x, int y, int cxWidth, int cyHeight,
		CWnd* pWndParent, void* pParam);

	virtual BOOL DestroyWindow();

	// special pre-creation and window rect adjustment hooks
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL PreCreateWindow_VisualInfo(XVisualInfo& vi);

	inline HWND GetSafeHwnd() const;
	inline BOOL IsChild(const CWnd* pWnd) const;
	inline void SetParent(CWnd* pWndNewParent);
	inline CWnd* GetParent() const;

	// Window Operations
	LRESULT SendMessage(UINT message, XEvent* pEvent);
	void SetWindowText(LPCTSTR pszString);
	int GetWindowText(LPTSTR pszStringBuf, int nMaxCount) const;
	void Invalidate(BOOL bErase =true);
	void UpdateWindow();
	bool ShowWindow(int nCmdShow);
	void MoveWindow(RECT* pRect, bool bRepaint =true);
	void MoveWindow(int x, int y, int nWidth, int nHeight,
		bool bRepaint =true);

	void ClientToScreen(POINT* pPoint) const;
	void ScreenToClient(POINT* pPoint) const;

	void SetCursor(HCURSOR hCursor);
	void SetCursorPos(int x, int y);
	void SetFocus();

	// Grab Mouse Pointer
	BOOL SetCapture();
	BOOL ReleaseCapture();
	static CWnd* PASCAL GetCapture();

//	static CWnd* PASCAL GetDesktopWindow();

protected:
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
//	virtual BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult);

// Nonclient-Area message handler member functions
//	afx_msg void OnNcDestroy();
	afx_msg void OnDestroy();
	
// Input message handler member functions
/*
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnDeadChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
	afx_msg int OnMouseActivate(CWnd* pDesktopWnd, UINT nHitTest, UINT message);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
//	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
//	afx_msg LRESULT OnRegisteredMouseWheel(WPARAM wParam, LPARAM lParam);
	afx_msg void OnRButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnTimer(UINT nIDEvent);
*/
//	afx_msg void OnLButtonDown(UINT nFlags, int x, int y);

	virtual LRESULT WindowProc(UINT message, XEvent* pEvent);
	virtual LRESULT DefWindowProc(UINT message, XEvent* pEvent);
	virtual BOOL OnXfcMsg(UINT nMsg, XEvent* pEvent, LRESULT* pResult);

	DECLARE_MESSAGE_MAP()

	CWnd* m_pParentWnd;	// parent window

public:
	static Atom s_wmDeleteWindow;

	friend class CWinApp;
};

///////////////////////////////////////////////////////////////////////////////

class CStatic : public CWnd
{
public:
	CStatic();

	bool Create(LPCTSTR pszText, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID =0xFFFF);

private:
	LPCTSTR m_pszText;
	DWORD m_dwStyle;

	DECLARE_MESSAGE_MAP()

	afx_msg void OnDraw(CDC* pDC);
};

class CButton : public CWnd
{
public:
	bool Create(LPCTSTR pszCaption, DWORD dwStyle, const RECT& rect,
		CWnd* pParentWnd, UINT nID);

protected:
	bool PtInButton(int x, int y);

	LPCTSTR m_pszCaption;
	DWORD m_dwStyle;
	UINT m_nID;

	int m_nItemXpos, m_nItemYpos;	// draw position
	int m_cxButton, m_cyButton;
	bool m_bButtonDown, m_bDrawSunken;

	DECLARE_MESSAGE_MAP()

	afx_msg void OnMouseMove(UINT nFlags, int x, int y);
	afx_msg void OnLButtonDown(UINT nFlags, int x, int y);
	afx_msg void OnLButtonUp(UINT nFlags, int x, int y);
	afx_msg void OnDraw(CDC* pDC);
};

///////////////////////////////////////////////////////////////////////////////

class CFrameWnd : public CWnd
{
public:
	CFrameWnd();
	virtual ~CFrameWnd();

	BOOL Create(LPCTSTR pszWindowName,
		DWORD dwStyle = 0,
		const RECT* pRect = NULL,
		CWnd* pParentWnd = NULL,
		void* pCreateParam = NULL);

	void SetMenu(CMenu* pMenu);

protected:
	SIZE m_sizeFrameWnd;

	CMenuBar* m_pMenuBar;
	CStatusBar* m_pStatusBar;
	CToolBar* m_pToolBar;

	DECLARE_MESSAGE_MAP()

	virtual void RecalcLayout(bool bNotify =true);
	virtual void OnSizeClient(RECT* pRectClient);

	afx_msg void OnClose();
	afx_msg void OnSize(int cx, int cy);

	friend class CStatusBar;
	friend class CToolBar;
};

class CDialog : public CWnd
{
public:
	CDialog();
//	CDialog(LPCTSTR lpszTemplateName, CWnd* pParentWnd =NULL);
//	CDialog(UINT nIDTemplate, CWnd* pParentWnd =NULL);
	virtual ~CDialog();

	BOOL Create(LPCTSTR lpszTemplateName, CWnd* pParentWnd =NULL);
	BOOL Create(UINT nIDTemplate, CWnd* pParentWnd =NULL);

	virtual int DoModal();
	void EndDialog(int nResult);

	virtual BOOL OnInitDialog();

	virtual void OnOK();
	virtual void OnCancel();

protected:
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);

	void Add(CWnd* pWnd);

	DECLARE_MESSAGE_MAP()

	afx_msg void OnSize(int cx, int cy);

private:
	bool IsMyChild(CWnd* pWnd);

	friend class CWinApp;
//	friend class CWnd;
};

///////////////////////////////////////////////////////////////////////////////
// Menu

class CMenuBar;

class CMenuItem
{
public:
	LPTSTR pszLabel;
	LPTSTR pszShortcut;
	UINT nFlags;
	int nStatus;
	union
	{
		UINT nID;
		CMenu* pSubMenu;
	} data;
private:
	int nHotPos;			// position of '&' of label
	BYTE nLabelLen;			// length of label string
	BYTE nShortcutLen;		// length of shortcut string

	friend class CMenu;
	friend class CMenuBar;
	friend class CPopupMenu;
};

typedef CSimplePtrList<CMenuItem*, CMenuItem*&> CMenuItemList;

class CMenuTracker;

class CMenu
{
public:
	CMenu();
	~CMenu();

	CMenu* GetSubMenu(int nPos) const;

	BOOL AppendMenu(UINT nFlags, UINT nID =0, LPCTSTR pszLabel =NULL);
	UINT CheckMenuItem(UINT nIDCheckItem, UINT nCheck);
	UINT EnableMenuItem(UINT nIDEnableItem, UINT nEnable);
	BOOL ModifyMenu(UINT nPosition, UINT nFlags, UINT nID =0, LPCTSTR pszLabel =NULL);
	void TrackPopupMenu(UINT nFlags, int x, int y, CWnd* pWnd);

protected:
	CMenuItemList m_listMenu;

	friend class CMenuBar;
	friend class CPopupMenu;
};

class CMenuBar : public CWnd
{
public:
	~CMenuBar();

	BOOL Create(CMenu* pMenu, CWnd* pParentWnd);

	int GetHeight() const;
	void SetWidth(int cx);

private:
	CMenuItemList* m_pMenuItemList;
	int m_nBarWidth, m_nBarHeight;		// size of this bar wnd

	// Item Layout Info
	struct MenubarItem
	{
		short left, top, right, bottom;	// box position
		CMenuItem* pItem;
	} *m_pMenubarItems, *m_pSelectedItem;
	int m_nMenubarItems;

	void RebuildLayout();
	void ReleaseTracking();
	struct MenubarItem *GetBarItemByPos(int x, int y);
	void DrawItem(CDC* pDC, struct MenubarItem* pBarItem, bool bSelect);

	bool m_bUnderTracking;
	CPopupMenu* m_pPopupMenu;	// share with all sub-popups


	DECLARE_MESSAGE_MAP()

	afx_msg void OnLButtonDown(UINT nFlags, int x, int y);
	afx_msg void OnLButtonUp(UINT nFlags, int x, int y);
	afx_msg void OnMouseMove(UINT nFlags, int x, int y);
	afx_msg void OnDraw(CDC* pDC);
	afx_msg void OnDestroy();

	friend class CMenuTracker;
};

class CPopupMenu : public CWnd
{
public:
	CPopupMenu();
	~CPopupMenu();

	BOOL Create(CWnd* pNotifyWnd, CWnd* pParentWnd, bool bStandAlone =false);
	void Show(int x, int y, CMenu* pMenu);
	void Hide();

private:
	CWnd *m_pNotifyWnd;

	CMenuItemList* m_pMenuItemList;	// content
	int m_nBoxWidth, m_nBoxHeight;	// size of popup-menu box
	bool m_bHidden;
	bool m_bStandAlone;		// called from TrackPopupMenu ?
	short m_nShortcutXPos;
	short m_nItemRectWidth;
	CPopupMenu* m_pSubPopupMenu;

	char m_nItemHeight, m_nFontYAdjust;	// non-changeable while app run

	struct PopupmenuItem
	{
		short top, bottom;	// box
		CMenuItem* pItem;
	} *m_pPopupmenuItems, *m_pSelectedItem;
	int m_nPopupmenuItems;

	void RebuildLayout();
	struct PopupmenuItem *GetPmItemByPos(int x, int y);
	void DrawItemText(CDC* pDC, int xx, int y, CMenuItem* pItem, XFontStruct* pFS);
	void DrawItem(CDC* pDC, struct PopupmenuItem* pPmItem, bool bSelect);

	DECLARE_MESSAGE_MAP()

	afx_msg void OnDraw(CDC* pDC);
	afx_msg void OnLButtonDown(UINT nFlags, int x, int y);
	afx_msg void OnLButtonUp(UINT nFlags, int x, int y);
	afx_msg void OnRButtonDown(UINT nFlags, int x, int y);
	afx_msg void OnRButtonUp(UINT nFlags, int x, int y);
	afx_msg void OnMouseMove(UINT nFlags, int x, int y);
	afx_msg void OnLeaveWindow();
	afx_msg void OnDestroy();

	friend class CMenu;
	friend class CMenuTracker;
};

class CMenuTracker
{
public:
	CMenuTracker(CMenuBar* pMB, CPopupMenu* pPM);
	~CMenuTracker();

	bool IsOurMessage(HWND hWnd);

private:
	CMenuBar* m_pMenuBar;
	CPopupMenu* m_pPopupMenu;
};

///////////////////////////////////////////////////////////////////////////////

class CControlBar : public CWnd
{
public:
};

///////////////////////////////////////////////////////////////////////////////

class CStbPaneInfo
{
public:
	UINT nID;
	UINT nStyle;
	int cxWidth;
	LPCTSTR pszText;
};

class CStatusBar : public CControlBar
{
public:
	CStatusBar();
	~CStatusBar();

	BOOL Create(CFrameWnd* pParentWnd, DWORD dwStyle =WS_VISIBLE, UINT nID=0);
	BOOL SetIndicators(const UINT *pIDArray, int nIDCount);

	void SetPaneInfo(int nIndex, UINT nID, UINT nStyle, int cxWidth);
	void SetPaneInfo(int nIndex, UINT nID, UINT nStyle, LPCTSTR pszText);
	BOOL SetPaneText(int nIndex, LPCTSTR pszNewText, BOOL bUpdate =TRUE);
	void SetPaneStyle(int nIndex, UINT nStyle);

// Positioning	
	int GetHeight() const;
	void SetWidth(int cx);

	int *m_pRightEdges;
protected:
	int m_nBarWidth;
	int m_nPaneCount;	// num of PaneInfos
	CStbPaneInfo* m_pPaneInfos;

	DECLARE_MESSAGE_MAP()

	afx_msg void OnDraw(CDC* pDC);
};

class CToolBar : public CControlBar
{
public:
	CToolBar();
	~CToolBar();

	BOOL Create(CFrameWnd* pParentWnd, DWORD dwStyle =WS_VISIBLE, UINT nID=0);
	BOOL LoadBitmap(char *xpmdata[]);
	BOOL SetButtons(const UINT *pIDArray, int nIDCount);
	void SetButtonInfo(int nIndex, UINT nID, UINT nStyle, int iImage);

// Positioning
	int GetHeight() const;
	void SetWidth(int cx);

private:
	int m_nBarWidth;
	Pixmap m_hPixmap;
	Pixmap m_hpmMask;

	// Button Layout Info
	struct BtnItem
	{
		short left, right;	// position
		TBBUTTON tbb;
	} *m_pBtnItems, *m_pSelectedItem;
	int m_nButtonCount;
	bool m_bButtonDown, m_bSunkenDrawn;

	struct BtnItem *GetBtnItemByPos(int x, int y);
	void DrawEdge(CDC* pDC, struct BtnItem* pItem, bool bDraw);

	DECLARE_MESSAGE_MAP()

	afx_msg void OnDraw(CDC* pDC);
	afx_msg void OnLButtonDown(UINT nFlags, int x, int y);
	afx_msg void OnLButtonUp(UINT nFlags, int x, int y);
	afx_msg void OnMouseMove(UINT nFlags, int x, int y);
	afx_msg void OnLeaveWindow();
};

///////////////////////////////////////////////////////////////////////////////

class CWinApp
{
public:
// Constructor
	CWinApp(LPCTSTR pszAppName =NULL);

	CWnd* m_pMainWnd;

// Overridables
	virtual BOOL InitInstance();
	virtual BOOL InitApplication();
	virtual int ExitInstance();
	virtual int Run();
	virtual ~CWinApp();

	void PostMessage(XFCMSG* pMsg);
	BOOL GetMessage(XFCMSG* pMsg);
	BOOL PeekMessage(XFCMSG* pMsg);
	BOOL PumpMessage();

private:
	BOOL TranslateMessage(XFCMSG* pMsg);
	void DispatchMessage(XFCMSG* pMsg);

	// Internal Message Queue buffer
#define XFC_MSGQUEUE_MAXSIZE		128
	XFCMSG m_msgQueue[XFC_MSGQUEUE_MAXSIZE], *m_pMsgCur;
	int m_nMsgQueueFront;		// front pointer
	int m_nMsgQueueRear;		// rear pointer

	// Menu Tracking
	CMenuTracker* m_pMenuTracker;
	void SetMenuTracker(CMenuTracker* pMT);
	friend class CMenu;
	friend class CMenuBar;
	friend class CPopupMenu;
	// Modal Dialog
	CDialog* m_pModalLeader;
	void AddModalLeader(CDialog* pModal);
	void RemoveModalLeader();
	friend class CDialog;
};

#include "xfc1.inl"

///////////////////////////////////////////////////////////////////////////////

int XfcMessageBox(LPCTSTR lpszText, UINT nType =MB_OK, UINT nIDHelp =0);
int XfcMessageBox(UINT nIDPrompt, UINT nType =MB_OK, UINT nIDHelp =(UINT)-1);
void XfcPostQuitMessage(int nExitCode);
void XfcPostMessage(XFCMSG* pMsg);

#endif
