// vim:ts=4
#ifndef __XFC_H__
#define __XFC_H__

#ifndef __cplusplus
	#error XFC requires C++ compilation (use a .cpp suffix)
#endif

#ifdef _WIN32

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <commdlg.h>
#include <shellapi.h>

#include "exttypes.h"

#else

#define PASCAL
#define CDECL
//#define PASCAL	__attribute__ ((stdcall))
//#define CDECL	__attribute__ ((cdecl))

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/xpm.h>
#include <unistd.h>

#include "wintypes.h"

#endif	// _WIN32

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define AFX_CDECL	CDECL
#define AFXAPI		PASCAL

/////////////////////////////////////////////////////////////////////////////
// Classes declared in this file
//	in addition to standard primitive data types and various helper macros

struct CRuntimeClass;

class CObject;
class CCmdTarget;
class CWinApp;
class CWnd;
class CMenu;
class CMenuBar;
class CDC;
class CStatusBar;
class CToolBar;

/////////////////////////////////////////////////////////////////////////////
// Sepcial definition values

#define ID_SEPARATOR		0

/////////////////////////////////////////////////////////////////////////////
// Basic types

// abstract iteration position
struct __POSITION { };
typedef __POSITION* POSITION;

// Standard constants
#undef FALSE
#undef TRUE
#undef NULL

#define FALSE	0
#define TRUE	1
#define NULL	0

/////////////////////////////////////////////////////////////////////////////
// Unicode support

#ifdef UNICODE
#define _T(s)	L##s
#else
#define _T(s)	s
#endif

/////////////////////////////////////////////////////////////////////////////
// Diagnostic support

#ifdef _DEBUG

#include <assert.h>

#define TRACE			::XfcTrace
#define THIS_FILE		__FILE__
#define ASSERT(f)		assert(f)
#define VERIFY(f)		ASSERT(f)
#define ASSERT_VALID(pOb)	ASSERT(pOb)
#define DEBUG_ONLY(f)		(f)

void AFX_CDECL XfcTrace(LPCTSTR lpszFormat, ...);

#define TRACE0(sz)              ::XfcTrace(_T("%s"), _T(sz))
#define TRACE1(sz, p1)          ::XfcTrace(_T(sz), p1)
#define TRACE2(sz, p1, p2)      ::XfcTrace(_T(sz), p1, p2)
#define TRACE3(sz, p1, p2, p3)  ::XfcTrace(_T(sz), p1, p2, p3)

inline BOOL AFX_CDECL AfxIsValidAddress(void* p, int size) { return true; }
#else   // _DEBUG

#define ASSERT(f)          ((void)0)
#define VERIFY(f)          ((void)(f))
#define ASSERT_VALID(pOb)  ((void)0)
#define DEBUG_ONLY(f)      ((void)0)
inline void AFX_CDECL XfcTrace(LPCTSTR, ...) { }
#define TRACE              1 ? (void)0 : ::XfcTrace
#define TRACE0(sz)
#define TRACE1(sz, p1)
#define TRACE2(sz, p1, p2)
#define TRACE3(sz, p1, p2, p3)

inline void AFX_CDECL AfxIsValidAddress(void* p, int size) { }
#endif // !_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Inline function declarations

#ifdef _AFX_ENABLE_INLINES
#define _AFX_INLINE inline
#endif

#ifndef afx_msg
#define afx_msg		// meaningless placeholder
#endif

///////////////////////////////////////////////////////////////////////////////
// Window message map handling

typedef void (CCmdTarget::*AFX_PMSG)(void);
typedef void (CWnd::*AFX_PMSGW)(void);

struct AFX_MSGMAP_ENTRY
{
	UINT nMessage;	// windows message
	UINT nCode;	// control code or WM_NOTIFY code
	UINT nID;	// control ID (or 0 for window messages)
	UINT nLastID;	// used for entries specifying a range of control id's
	UINT nSig;	// signature type (action) or pointer to message #
	AFX_PMSG pfn;	// routine to call (or special value)
};

struct AFX_MSGMAP
{
	const AFX_MSGMAP* pBaseMap;
	const AFX_MSGMAP_ENTRY* pEntries;
};

namespace global_xfc
{
}

#define DECLARE_MESSAGE_MAP() \
private: \
	static const AFX_MSGMAP_ENTRY _messageEntries[]; \
protected: \
	static const AFX_MSGMAP messageMap; \
	static const AFX_MSGMAP* _GetBaseMessageMap(); \
	virtual const AFX_MSGMAP* GetMessageMap() const; \

#define BEGIN_MESSAGE_MAP(theClass, baseClass) \
	const AFX_MSGMAP* theClass::GetMessageMap() const \
		{ return &theClass::messageMap; } \
	const AFX_MSGMAP theClass::messageMap = \
	{ &baseClass::messageMap, &theClass::_messageEntries[0] }; \
	const AFX_MSGMAP_ENTRY theClass::_messageEntries[] = {

#define END_MESSAGE_MAP() \
		{0, 0, 0, 0, XfcSig_end, (AFX_PMSG)0 } \
	};

// Message map signature values and macros in separate header
#include "msgsig_.h"

///////////////////////////////////////////////////////////////////////////////

class CRect : public tagRECT
{
public:
	BOOL PtInRect(POINT point) const;
	BOOL PtInRect(int x, int y) const;
	BOOL EqualRect(RECT* pRect) const;
	void CopyRect(RECT* pSrcRect);

// Operations
	void SetRect(int x1, int y1, int x2, int y2);
	void InflateRect(int x, int y);
};

///////////////////////////////////////////////////////////////////////////////

#include "SimplePtrList.h"

#ifdef _WIN32
#include "win32/xfcwin.h"
#else
#include "x11/xfcwin.h"
#endif

#endif	// __AFX_H__
